
__kernel void computeDensity
(
  __global float3* pos, 
  __global float3* vel, 
  __global float3* vorticity,
  __global float* density, 
  const int useVorticity,
  __global int* neighboursFluidFluid,
  const int neighboursFluidFluidCountMax,
  __global float3* solidPos,  
  __global float* solidVolume, 
  __global int* neighboursFluidSolid,
  const int neighboursFluidSolidCountMax
) 
{
  int i = get_global_id(0);
  int fluidStart = i*neighboursFluidFluidCountMax;
  int fluidEnd = fluidStart + neighboursFluidFluidCountMax;
  int solidStart = i*neighboursFluidSolidCountMax;
  int solidEnd = solidStart + neighboursFluidSolidCountMax;
  
  float3 posi = pos[i];
  float3 veli = vel[i];
  float3 vec;
  
  int j;
  
  float r2, t, t2, t3, mt2;
  float3 posj;
  float3 vorticityTerm;
  
  float densityTmp = density[i];
  float3 vorticityTmp = vorticity[i];
  
  // Fluid-fluid loop
  for (int k = fluidStart; k < fluidEnd; k++) {
    j = neighboursFluidFluid[k];
    
    if (j > -1) {
      posj = pos[j];
      vec = posi - posj;
    
      r2 = length2v(vec); 

		  t = 1 - r2 * invRadius2;
      t2 = t*t;
			t3 = t2*t;

      // DENSITY
			densityTmp += mass0 * t3;
      
      // VORTICITY
      if (useVorticity) {
        vorticityTerm = cross(vel[j] - veli, vec);
        mt2 = mass0 * t2;
        vorticityTmp += mt2 * vorticityTerm;  
      }
    }
    else {
      break;
    }
  }
  
  // Fluid-solid loop
  for (int k = solidStart; k < solidEnd; k++) {
    j = neighboursFluidSolid[k];
    
    if (j > -1) {
      vec = posi - solidPos[j];
    
      r2 = length2v(vec); 

		  t = 1 - r2 * invRadius2;
      t2 = t*t;
			t3 = t2*t;

      // DENSITY
      densityTmp += solidVolume[j] * t3;
    }
    else {
      break;
    }
  }
    
  // DENSITY
  densityTmp = wDensity * (densityTmp + mass0);
  
  // VORTICITY
  if (useVorticity) {
    vorticityTmp = dWDensity / densityTmp * vorticityTmp;
  }
  
  // Final sum
  density[i] = densityTmp;
  vorticity[i] = vorticityTmp;
}
