
__kernel void computePressureForce
(
  __global float3* pos, 
  __global float* density, 
  __global float* pressure, 
  __global float3* pressureForce, 
  __global int* neighboursFluidFluid,
  const int neighboursFluidFluidCountMax,
  __global float3* solidPos,  
  __global float* solidVolume, 
  __global int* neighboursFluidSolid,
  const int neighboursFluidSolidCountMax
) 
{
  int i = get_global_id(0);
  int fluidStart = i*neighboursFluidFluidCountMax;
  int fluidEnd = fluidStart + neighboursFluidFluidCountMax;
  int solidStart = i*neighboursFluidSolidCountMax;
  int solidEnd = solidStart + neighboursFluidSolidCountMax;
  
  float3 posi = pos[i];
  
  float pressurei = pressure[i];
  float densityi = density[i];
  
  int j;
  
  float r, r2, t, tp;
  float3 vec;
  
  float pressureDensity = -pressurei / (densityi * densityi) * dWPressure;
  
  float3 pressureForceTmp = pressureForce[i];
  
  // Fluid-fluid loop
  for (int k = fluidStart; k < fluidEnd; k++) {
    j = neighboursFluidFluid[k];
    
    if (j > -1) {
      vec = posi - pos[j];
    
      r2 = length2v(vec); 

		  r = sqrt(r2);
			t = 1 - r * invRadius;

      tp = mass0 * (-0.5 * (pressurei + pressure[j]) / (densityi * density[j]) * dWPressure * t * t / (r /*+ 0.0001*/));

			pressureForceTmp += tp * vec;
    }
    else {
      break;
    }
  }
  
  // Fluid-solid loop
  for (int k = solidStart; k < solidEnd; k++) {
    j = neighboursFluidSolid[k];
    
    if (j > -1) {
      vec = posi - solidPos[j];
    
      r2 = length2v(vec); 

		  r = sqrt(r2);
			t = 1 - r * invRadius;

			tp = solidVolume[j] * (pressureDensity * t * t / r);

			pressureForceTmp += tp * vec;
    }
    else {
      break;
    }
  }
  
  // Final sum
  pressureForce[i] = pressureForceTmp;
}
