
__kernel void computeSumA
(
  __global float3* pos, 
  __global float3* d_ii, 
  __global float* a_ii, 
  __global float3* d_ijp_j,
  __global float* sumA, 
  __global float* density, 
  __global float* pressure, 
  __global int* neighboursFluidFluid,
  const int neighboursFluidFluidCountMax,
  __global float3* solidPos,   
  __global float* solidVolume, 
  __global int* neighboursFluidSolid,
  const int neighboursFluidSolidCountMax
) 
{
  int i = get_global_id(0);
  
  if (fabs(a_ii[i]) >= 1e-10) {
    int fluidStart = i*neighboursFluidFluidCountMax;
    int fluidEnd = fluidStart + neighboursFluidFluidCountMax;
    int solidStart = i*neighboursFluidSolidCountMax;
    int solidEnd = solidStart + neighboursFluidSolidCountMax;
    
    float3 posi = pos[i];
    
    float3 d_ijp_ji = d_ijp_j[i];
    float pressurei = pressure[i];
    
    int j;
    
    float r, t, tp, dtp;
    float3 vec;
    
    float density2 = density[i] * density[i];
    float dterm = deltaTime2 * mass0 / density2;
    float3 d_ji;
    
    float sumATmp = sumA[i];
    
    // Fluid-fluid loop
    for (int k = fluidStart; k < fluidEnd; k++) {
      j = neighboursFluidFluid[k];
      
      if (j > -1) {
        vec = posi - pos[j];
      
        r = length(vec); 
			  t = 1 - r * invRadius;

        tp = dWPressure * t * t / r;
        dtp = dterm * tp;
        
        d_ji = dtp * vec;
				
        sumATmp += mass0 * tp * dot(d_ijp_ji - d_ii[j] * pressure[j] - d_ijp_j[j] + d_ji * pressurei, vec);
      }
      else {
        break;
      }
    }
    
    // Fluid-solid loop
    for (int k = solidStart; k < solidEnd; k++) {
      j = neighboursFluidSolid[k];
      
      if (j > -1) {
        vec = posi - solidPos[j];
      
        r = length(vec); 
			  t = 1 - r * invRadius;

        tp = dWPressure * t * t / r;
     
        sumATmp += solidVolume[j] * tp * dot(d_ijp_ji, vec);
      }
      else {
        break;
      }
    }
    
    // Final sum
    sumA[i] = sumATmp;
  }
}
