
__kernel void predictVelocity
(
  __global float3* pos, 
  __global float3* vel,  
  __global float3* acc,  
  __global float* density, 
  __global float* pressure, 
  __global float3* velocityPrediction, 
  __global float3* d_ii, 
  __global int* neighboursFluidFluid,
  const int neighboursFluidFluidCountMax,
  __global float3* solidPos, 
  __global float* solidVolume, 
  __global int* neighboursFluidSolid,
  const int neighboursFluidSolidCountMax
) 
{
  int i = get_global_id(0);
  int fluidStart = i*neighboursFluidFluidCountMax;
  int fluidEnd = fluidStart + neighboursFluidFluidCountMax;
  int solidStart = i*neighboursFluidSolidCountMax;
  int solidEnd = solidStart + neighboursFluidSolidCountMax;
  
  float3 posi = pos[i];
  
  float densityi = density[i];
  
  int j;
  
  float r, t, tp;
  float3 vec;
  
  float density2 = densityi * densityi;
  float massDensity2 = mass0 / density2;
  
  velocityPrediction[i] = vel[i] + (deltaTime * acc[i]);
  
  pressure[i] *= 0.5;
  
  float3 d_iiTmp = d_ii[i];
  
  // Fluid-fluid loop
  for (int k = fluidStart; k < fluidEnd; k++) {
    j = neighboursFluidFluid[k];
    
    if (j > -1) {
      vec = posi - pos[j];
    
      r = length(vec); 
			t = 1 - r * invRadius;
      
      tp = massDensity2 * (dWPressure * t * t / r);
      
			d_iiTmp -= tp * vec;
    }
    else {
      break;
    }
  }
  
  // Fluid-solid loop
  for (int k = solidStart; k < solidEnd; k++) {
    j = neighboursFluidSolid[k];
    
    if (j > -1) {
      vec = posi - solidPos[j];
    
      r = length(vec); 
			t = 1 - r * invRadius;
      
      tp = solidVolume[j] / density2 * (dWPressure * t * t / r);

			d_iiTmp -= tp * vec;
    }
    else {
      break;
    }
  }

  d_iiTmp *= deltaTime2;
  
  // Final sum
  d_ii[i] = d_iiTmp;
}
