#include "clframework.h"

using namespace clframework;

CLConstant::CLConstant(std::string name, CLVariableDataType varDataType):
  CLKernelArgument(name, varDataType) {

}

CLConstant::CLConstant(std::string name, CLVariableDataType varDataType, double var):
  CLKernelArgument(name, varDataType) {

  CLVariable::setScalar(var);
}

CLConstant::CLConstant(std::string name, CLVariableDataType varDataType, Vector<3, double> var): 
  CLKernelArgument(name, varDataType) {

  CLVariable::setVector(var);
}

CLConstant::~CLConstant() {

}

std::string CLConstant::getSource() {
  if (data == NULL) {
    return "";
  }

	std::ostringstream stream;
	Vector<2, float> v2f;

	switch (varDataType) {
	  case FloatType:
      stream << "(float)" << *(float*)data; break;
	  case DoubleType:
      stream << "(double)" << *(double*)data; break;
	  case IntType:
      stream << "(int)" << *(int*)data; break;
	  case UIntType:
      stream << "(uint)" << *(unsigned int*)data; break;
    case LongType:
      stream << "(long)" << *(long long*)data; break;
    case ULongType:
      stream << "(ulong)" << *(unsigned long long*)data; break;
	  case Float2Type:
		  stream << "(float2)(" << ((Vector<2,float>*)data)->x << ", " << ((Vector<2,float>*)data)->y << ")"; break;
	  case Double2Type:
		  stream << "(double2)(" << ((Vector<2,double>*)data)->x << ", " << ((Vector<2,double>*)data)->y << ')'; break;
	  case Int2Type:
		  stream << "(int2)(" << ((Vector<2,int>*)data)->x << ", " << ((Vector<2,int>*)data)->y << ')'; break;
	  case UInt2Type:
		  stream << "(uint2)(" << ((Vector<2,unsigned int>*)data)->x << "u, " << ((Vector<2,unsigned int>*)data)->y << "u)"; break;
    case Long2Type:
      stream << "(long2)(" << ((Vector<2, long long>*)data)->x << ", " << ((Vector<2, long long>*)data)->y << ")"; break;
    case ULong2Type:
      stream << "(ulong2)(" << ((Vector<2, unsigned long long>*)data)->x << "u, " << ((Vector<2, unsigned long long>*)data)->y << "u)"; break;
    case Float3Type:
      stream << "(float3)(" << ((Vector<3, float>*)data)->x << ", " << ((Vector<3, float>*)data)->y << ", " << ((Vector<3, float>*)data)->z << ")"; break;
    case Double3Type:
      stream << "(double3)(" << ((Vector<3, double>*)data)->x << ", " << ((Vector<3, double>*)data)->y << ", " << ((Vector<3, float>*)data)->z << ')'; break;
    case Int3Type:
      stream << "(int3)(" << ((Vector<3, int>*)data)->x << ", " << ((Vector<3, int>*)data)->y << ", " << ((Vector<3, float>*)data)->z << ')'; break;
    case UInt3Type:
      stream << "(uint3)(" << ((Vector<3, unsigned int>*)data)->x << "u, " << ((Vector<3, unsigned int>*)data)->y << "u, " << ((Vector<3, float>*)data)->z << "u)"; break;
   case Float4Type:
      stream << "(float4)(" << ((Vector<4, float>*)data)->x << ", " << ((Vector<4, float>*)data)->y << ", " << ((Vector<4, float>*)data)->z << ", " << ((Vector<4, float>*)data)->w << ")"; break;
	  case Double4Type:
      stream << "(double4)(" << ((Vector<4, double>*)data)->x << ", " << ((Vector<4, double>*)data)->y << ", " << ((Vector<4, double>*)data)->z << ", " << ((Vector<4, double>*)data)->w << ")"; break;
	  case Int4Type:
      stream << "(int4)(" << ((Vector<4, int>*)data)->x << ", " << ((Vector<4, int>*)data)->y << ", " << ((Vector<4, int>*)data)->z << ", " << ((Vector<4, int>*)data)->w << ")"; break;
	  case UInt4Type:
      stream << "(uint4)(" << ((Vector<4, unsigned int>*)data)->x << "u, " << ((Vector<4, unsigned int>*)data)->y << "u, " << ((Vector<4, unsigned int>*)data)->z << "u, " << ((Vector<4, unsigned int>*)data)->w << "u)"; break;
	  case CharType:
		  stream << "(char)(" << *(char*)data; break;
	  case UCharType:
		  stream << "(uchar)(" << *(unsigned char*)data; break;
	  default:
		  return "";
	}

	return stream.str();
}
