#include "clframework.h"

using namespace clframework;

CLDevice::CLDevice(CLPlatform* platform, cl_device_id id):
  platform(platform),
  id(id),
  type(CL_DEVICE_TYPE_DEFAULT),
  maxComputeUnits(0),
  maxClock(0),
  maxWorkGroupSize(0),
  globalMemorySize(0),
  localMemorySize(0),
  maxAllocationSize(0),
  fp16(false),
  fp64(false),
  globalAtomics(false),
  localAtomics(false),
  clGlSharing(false) {

}

CLDevice::~CLDevice() {

}

void CLDevice::initialize() {
  Log* log = Log::getInstance();

  char buf[4096];
  size_t size;

  cl_int status = clGetDeviceInfo(id, CL_DEVICE_TYPE, sizeof(cl_device_type), &type, NULL);
  status = clGetDeviceInfo(id, CL_DEVICE_VERSION, sizeof(cl_device_type), &type, NULL);
  status = clGetDeviceInfo(id, CL_DEVICE_MAX_COMPUTE_UNITS, sizeof(cl_uint), &maxComputeUnits, NULL);
  status = clGetDeviceInfo(id, CL_DEVICE_MAX_CLOCK_FREQUENCY, sizeof(cl_uint), &maxClock, NULL);
  status = clGetDeviceInfo(id, CL_DEVICE_MAX_WORK_GROUP_SIZE, sizeof(size_t), &maxWorkGroupSize, NULL);
  status = clGetDeviceInfo(id, CL_DEVICE_MAX_WORK_ITEM_SIZES, sizeof(maxWorkItemSize), &maxWorkItemSize, NULL);
  status = clGetDeviceInfo(id, CL_DEVICE_GLOBAL_MEM_SIZE, sizeof(cl_ulong), &globalMemorySize, NULL);
  status = clGetDeviceInfo(id, CL_DEVICE_LOCAL_MEM_SIZE, sizeof(cl_ulong), &localMemorySize, NULL);
  status = clGetDeviceInfo(id, CL_DEVICE_MAX_MEM_ALLOC_SIZE, sizeof(cl_ulong), &maxAllocationSize, NULL);

  status = clGetDeviceInfo(id, CL_DEVICE_NAME, sizeof(buf), buf, &size);
  name = std::string(buf, size);
  
  status = clGetDeviceInfo(id, CL_DEVICE_VENDOR, sizeof(buf), buf, &size);
  vendor = std::string(buf, size);

  status = clGetDeviceInfo(id, CL_DEVICE_EXTENSIONS, sizeof(buf), buf, &size);
  std::string extensions(buf, size);

  fp64 = (extensions.find("cl_khr_fp64") != std::string::npos);
  fp16 = (extensions.find("cl_khr_fp16") != std::string::npos);

  globalAtomics = (extensions.find("cl_khr_global_int32_base_atomics") != std::string::npos && extensions.find("cl_khr_global_int32_extended_atomics") != std::string::npos);
  localAtomics = (extensions.find("cl_khr_local_int32_base_atomics") != std::string::npos && extensions.find("cl_khr_local_int32_extended_atomics") != std::string::npos);

  clGlSharing = (extensions.find("cl_khr_gl_sharing") != std::string::npos);

  log->writeLine(CLInfo, "DEVICE " + CLSystem::deviceTypeStringShort(type) + " for OpenCL found: " + name);
  log->writeLine(CLInfo, "Vendor: " + vendor);
  log->writeLine(CLInfo, "Extensions: " + extensions);
}