#include <CL/cl.h>

namespace clframework {

  class CLPlatform;

  class CLDevice {

    private:

      CLPlatform* platform;

      cl_device_id id;
      std::string name;
      std::string vendor;
      cl_device_type type;

      cl_uint maxComputeUnits;
      cl_uint maxClock;

      size_t maxWorkGroupSize;
      size_t maxWorkItemSize[3];
      cl_ulong globalMemorySize;
      cl_ulong localMemorySize;
      cl_ulong maxAllocationSize;

      bool fp16;
      bool fp64;
      bool globalAtomics;
      bool localAtomics;
      bool clGlSharing;

    public:

      CLDevice(CLPlatform* platform, cl_device_id id);
      ~CLDevice();

      void initialize();
      
      CLPlatform* getPlatform() { return platform; };

      cl_device_id& getId() { return id; }
      const std::string& getName() { return name; }
      const std::string& getVendor() { return vendor; }
      cl_device_type getType() { return type; };
      std::string getTypeString();
      bool isCPU() { return type == CL_DEVICE_TYPE_CPU; };
      bool isGPU() { return type == CL_DEVICE_TYPE_GPU; };

      unsigned int getMaxComputeUnits() { return (unsigned int)maxComputeUnits; }
      unsigned int getMaxClock() { return (unsigned int)maxClock; }
      
      unsigned int getGlobalMemorySize() { return (unsigned int)globalMemorySize; }
      unsigned int getLocalMemorySize() { return (unsigned int)localMemorySize; }
      unsigned int getMaxWorkGroupSize() { return (unsigned int)maxWorkGroupSize; }
      unsigned int getMaxWorkItemSize(unsigned int dimension) { return (unsigned int)maxWorkItemSize[dimension]; }
      unsigned int getMaxAllocationSize() { return (unsigned int)maxAllocationSize; }

      bool getDoublePrecision() { return fp64; }
      bool getHalfPrecision() { return fp16; }
      bool getAtomics() { return globalAtomics && localAtomics; }
      bool getGlobalAtomics() { return globalAtomics; }
      bool getLocalAtomics() { return localAtomics; }

  };

}