#include <CL/cl.h>

namespace clframework {

  class CLSystem;
  class CLDevice;
  class CLProgram;

  class CLGlobalBuffer: public CLVariable {

    protected:

      CLLink* link;

      bool isDynamic;

      unsigned int bufferCount;
      cl_mem* clBuffers;
      size_t* partElementCount;
      size_t* offsets;

      bool hostHasData;
      bool hostDataChanged;

      bool isSplit() { return bufferCount > 1; }
      size_t getOffset(unsigned int deviceID) { return isSplit() ? offsets[deviceID] : 0; }

      bool allocateHostData();

      virtual bool allocate();
      virtual void release();

      virtual bool setAsArgument(CLProgram* program, unsigned int argID, unsigned int deviceID = 0, size_t kernelLocalSize = 0);
      virtual size_t getElementCount(unsigned int deviceID) { return isSplit() ? partElementCount[deviceID] : elementCount; }

    public:

      CLGlobalBuffer(CLLink* link, std::string name, CLVariableDataType varDataType, unsigned int elements = 1, bool isDynamic = false, bool paralelize = false);
      virtual ~CLGlobalBuffer();

      CLLink* getLink() { return link; };

      void* getHostData() { return data; }
      bool getHostHasData() { return hostHasData; }
      bool getHostDataChanged() { return hostDataChanged; }

      bool download(bool waitToFinish = true, bool forceDownload = true);
      bool upload(bool waitToFinish = true);
      bool copyFrom(CLGlobalBuffer* var, bool waitToFinish = true);

      void reallocate(unsigned int elements);

      bool getIsDynamic() { return isDynamic; };

      virtual double getScalar(unsigned int id = 0);
      virtual Vector<3, double> getVector(unsigned int id = 0);

      virtual bool setScalar(unsigned int id, double var);
      bool setScalar(double var) { return setScalar(0, var); }

      virtual bool setVector(unsigned int id, Vector<3, double> var);
      bool setVector(Vector<3, double> var) { return setVector(0, var); }

      cl_mem getBuffer(unsigned int device) { return clBuffers[device]; }

  };

}