#include "clframework.h"

using namespace clframework;

CLKernelArgument::CLKernelArgument(std::string name, CLVariableDataType varDataType, unsigned int elements, bool paralelize) :
  CLVariable(name, KernelArgument, varDataType, elements, paralelize) {

  needsUpdate = false;
  allocate();
}

CLKernelArgument::~CLKernelArgument() {
  release();
}


bool CLKernelArgument::allocate() {
  release();

  data = new char[memorySize]();
  needsUpdate = false;

  return true;
}


bool CLKernelArgument::setAsArgument(CLProgram* program, unsigned int argID, unsigned int deviceID, size_t kernelLocalSize) {
  Log* log = Log::getInstance();

  log->writeLine(CLDebug, "Setting kernel argument " + name);

  cl_int status = clSetKernelArg(program->getKernel(), (cl_uint)argID, CLSystem::dataTypeSize(varDataType), data);

  if (status != CL_SUCCESS) {
    log->writeLine(CLError, "Error while setting kernel argument: " + CLSystem::statusDesc(status));
    return false;
  }

  return true;
}


void CLKernelArgument::release() {
  if (data != NULL) {
    delete[] data;
    data = NULL;
  }
}
