#include "clframework.h"

using namespace clframework;

CLLocalBuffer::CLLocalBuffer( std::string name, CLVariableDataType varDataType, unsigned int elements, bool paralelize):
  CLVariable(name, LocalBuffer, varDataType, elements, paralelize) {

  needsUpdate = true;
}

CLLocalBuffer::~CLLocalBuffer() {

}

bool CLLocalBuffer::allocate() {
  Log* log = Log::getInstance();

  release();

  log->writeLine(CLDebug, "Allocating on device buffer: " + name + ", note that for local buffer is no memory allocated");

  needsUpdate = false;

  return true; 
}

bool CLLocalBuffer::setAsArgument(CLProgram* program, unsigned int argID, unsigned int deviceID, size_t kernelLocalSize) {
  Log* log = Log::getInstance();

  log->writeLine(CLDebug, "Setting local buffer kernel argument " + name);

  if (program == NULL) {
    log->writeLine(CLError, "Cannot set local buffer kernel argument to NULL program");
    return false;
  }

  if (program->getKernel() == NULL) {
    log->writeLine(CLError, "Cannot set local buffer kernel argument to NULL kernel");
    return false;
  }

  cl_int status = clSetKernelArg(program->getKernel(), (cl_uint)argID, memorySize, NULL);

  if (status != CL_SUCCESS) {
    log->writeLine(CLError, "Setting local buffer kernel argument " + name + " failed: " + CLSystem::statusDesc(status));
    return false;
  }

  return true;
}

void CLLocalBuffer::release() {

}
