#include "clframework.h"

using namespace clframework;

CLPlatform::CLPlatform(cl_platform_id id):
  id(id) {

}

CLPlatform::~CLPlatform() {
  for (unsigned int i = 0; i < devices.size(); i++) {
    delete devices[i];
  }
}

void CLPlatform::initialize() {
  Log* log = Log::getInstance();

  char buf[4096];
  size_t size;

  clGetPlatformInfo(id, CL_PLATFORM_NAME, sizeof(buf), buf, &size);
  name = std::string(buf, size);
  
  clGetPlatformInfo(id, CL_PLATFORM_VENDOR, sizeof(buf), buf, &size);
  vendor = std::string(buf, size);

  clGetPlatformInfo(id, CL_PLATFORM_VERSION, sizeof(buf), buf, &size);
  clVersion = std::string(buf, size);

  cl_device_id deviceIDs[32];
  cl_uint deviceCount;
  cl_int status = clGetDeviceIDs(id, CL_DEVICE_TYPE_ALL, 32, deviceIDs, &deviceCount);
  
  log->writeLine(CLInfo, "PLATFORM for OpenCL found: " + name);
  std::ostringstream ss;
  ss << deviceCount;
  log->writeLine(CLInfo, "Device count: " + ss.str());
  log->writeLine(CLInfo, "Vendor: " + vendor);
  log->writeLine(CLInfo, "OpenCL version: " + clVersion);

  for (unsigned int i = 0; i < deviceCount; i++) {
    CLDevice* device = new CLDevice(this, deviceIDs[i]);
    device->initialize();

    devices.push_back(device);
  }
}

std::vector<CLDevice*> CLPlatform::getDevices(cl_device_type type) {
  std::vector<CLDevice*> selectedDevices;

  for (unsigned int i = 0; i < devices.size(); i++) {
    if (devices[i]->getType() == type || type == CL_DEVICE_TYPE_ALL) {
      selectedDevices.push_back(devices[i]);
    }
  }

  return selectedDevices;
}