#include <CL/cl.h>

namespace clframework {

  class CLProgram {

    private:

      CLLink* link;

      cl_program program;
      cl_kernel kernel;

      size_t* workGroupSizes;

      bool hasDynamicGlobalBuffer;

      bool isBuilt;
      bool unsafeMath;
      std::vector<std::string> buildOptions;
      std::string name;

      CLFunction* kernelSource;
      std::map<std::string, CLFunction*> functionsSources;

      std::map<CLVariable*, unsigned int> variablesArgIDs;
      std::map<std::string, CLVariable*> variables;
      std::map<std::string, CLKernelArgument*> kernelArguments;
      std::map<std::string, CLGlobalBuffer*> globalBuffers;
      std::map<std::string, CLLocalBuffer*> localBuffers;
      std::map<std::string, CLConstant*> constants;

      bool createKernel();
      void releaseKernel();

      void releaseProgram();

      bool setPersistentArguments();

    public:

      CLProgram(std::string sourcePath, std::string name);
      CLProgram(std::string sourcePath, std::string name, CLLink* link);
      ~CLProgram();

      CLLink* getLink() { return link; };
      cl_program getProgram() { return program; };
      cl_kernel getKernel() { return kernel; };
      std::string getName() { return name; };
      bool getIsBuilt() { return isBuilt; };
      
      void setUnsafeMath(bool value);
      void addBuildOption(const std::string& option);
      void clearBuildOptions();

      void addFunction(CLFunction* function);

      void addKernelArgument(CLKernelArgument* kernelArgument, unsigned int argID);
      void addGlobalBuffer(CLGlobalBuffer* globalBuffer, unsigned int argID);
      void addLocalBuffer(CLLocalBuffer* localBuffer, unsigned int argID);
      void addConstant(CLConstant* constant);

      CLKernelArgument* getKernelArgument(std::string name);
      CLGlobalBuffer* getGlobalBuffer(std::string name);
      CLLocalBuffer* getLocalBuffer(std::string name);

      bool setLink(CLLink* linkToDevices);
      bool build();
      bool enqueue(size_t globalSize = 0, size_t localSize = 0);
      bool finish();

      std::string getCompiledBinary();

  };

}