#include <CL/cl.h>

namespace clframework {

  enum CLVariableDataType {
    FloatType,		    // 32bit floating precision scalar
    Float2Type,		    // 32bit 2D floating precision vector
    Float3Type,		    // 32bit 3D floating precision vector
    Float4Type,		    // 32bit 4D floating precision vector
    DoubleType,		    // 64bit floating precision scalar
    Double2Type,	    // 64bit 2D floating precision vector
    Double3Type,	    // 64bit 3D floating precision vector
    Double4Type,	    // 64bit 4D floating precision vector
    UIntType,		      // 32bit positive integer
    UInt2Type,		    // 32bit 2D positive integer vector
    UInt3Type,		    // 32bit 3D positive integer vector
    UInt4Type,		    // 32bit 4D positive integer vector
    LongType,         // 64bit integer
    ULongType,        // 64bit positive integer
    Long2Type,        // 64bit 2D integer vector
    ULong2Type,       // 64bit 2D positive integer vector
    IntType,		      // 32bit integer
    Int2Type,		      // 32bit 2D integer vector
    Int3Type,		      // 32bit 3D integer vector
    Int4Type,		      // 32bit 4D integer vector
    CharType,		      // 8bit integer
    UCharType		      // 8bit positive integer
  };

  enum CLVariableType {
    KernelArgument,		// OpenCL variable type that can be used as kernel argument
    GlobalBuffer,		  // OpenCL global buffer
    LocalBuffer			  // OpenCL local buffer
  };

  class CLPlatform;

  class CLSystem {

    private:

      static CLSystem* clsystemInstance;

      std::vector<CLPlatform*> platforms;
      bool profiling;

      void initialize();

      CLSystem();

    public:

      ~CLSystem();

      unsigned int getPlatformCount() { return platforms.size(); }
      CLPlatform* getPlatform(unsigned int i) { return platforms[i]; }
      CLPlatform* getFirstPlatform() { return (platforms.size() > 0) ? platforms[0] : NULL; }

      bool getProfiling() { return profiling; };
      void setProfiling(bool value) { profiling = value; }

      static std::string statusDesc(cl_int status);
      static std::string deviceTypeString(cl_device_type type);
      static std::string deviceTypeStringShort(cl_device_type type);
      static size_t dataTypeSize(CLVariableDataType varDataType);

      static CLSystem* getInstance();

  };

}