#include <CL/cl.h>

namespace clframework {

  class CLSystem;
  class CLDevice;
  class CLLink;
  class CLProgram;

  class CLVariable {

    protected:

      friend class CLProgram;

      CLVariableType varType;
      CLVariableDataType varDataType;
      size_t varDataTypeSize;

      std::string name;

      char* data;
      size_t elementCount;
      size_t memorySize;
      bool needsUpdate;

      bool paralelize;

      virtual bool allocate() = 0;
      virtual void release() = 0;

      virtual bool setAsArgument(CLProgram* program, unsigned int argID, unsigned int deviceID = 0, size_t kernelLocalSize = 0) = 0;
      virtual size_t getElementCount(unsigned int deviceID) = 0;

      void* get(unsigned int id);

      CLVariable(std::string name, CLVariableType varType, CLVariableDataType varDataType, unsigned int elements = 1, bool paralelize = false);
      virtual ~CLVariable();

    public:

      CLVariableType getVarType() { return varType; };
      CLVariableDataType getVarDataType() { return varDataType; };

      std::string getName() { return name; };

      char* getData() { return data; };
      size_t getElementCount() { return elementCount; };
      size_t getMemorySize() { return memorySize; };
      bool getNeedsUpdate() { return needsUpdate; };

      bool getParalelize() { return paralelize; };
      void setParalelize(bool value) { paralelize = value; };

      virtual double getScalar(unsigned int id = 0);
      virtual Vector<3, double> getVector(unsigned int id = 0);

      virtual bool setScalar(unsigned int id, double var);
      bool setScalar(double var) { return setScalar(0, var); };

      virtual bool setVector(unsigned int id, Vector<3, double> var);
      bool setVector(Vector<3, double> var) { return setVector(0, var); };

      bool isScalar();

  };

}