#include <cmath>

namespace clframework {

  template<int dim, typename typ> struct Vector {

  };


  template<typename typ> struct Vector<2, typ> {

    public:

      typ x, y;

      Vector() {
        x = y = 0;
      }

      Vector(typ v) {
        x = y = v;
      }

      Vector(typ X, typ Y) {
        x = X; y = Y;
      }

      Vector(typ X, typ Y, typ Z) {
        x = X; y = Y;
      }

      Vector(typ* v) {
        x = v[0]; y = v[1];
      }

    template<int dim_, typename typ_> Vector<2, typ>& operator = (const Vector<dim_, typ_>& other) {
      x = (typ)other.x;
      y = (typ)other.y;
      return *this;
    }

    template<int dim_, typename typ_> operator Vector<dim_, typ_>() {
      return Vector<dim_, typ_>(static_cast<typ_>(x), static_cast<typ_>(y));
    }

    inline typ& operator[](unsigned int id) {
      return id ? y : x; //return (&x)[id];
    }

    inline typ dot(const Vector &v) const {
      return x*v.x + y*v.y;
    }

    inline typ lengthSq() const {
      return x*x + y*y;
    }

    inline typ length() const {
      return sqrt(lengthSq());
    }

    inline Vector normalize() const {
      return Vector(x, y) / length();
    }

    inline Vector rotate(double angle) const {
      return Vector(x*cos(angle) - y*sin(angle), y*cos(angle) + x*sin(angle));
    }

    inline const Vector operator-(void) const {
      return Vector(-x, -y);
    }

    inline Vector operator+(const Vector &v) const {
      return Vector(x + v.x, y + v.y);
    }

    inline Vector operator-(const Vector &v) const {
      return Vector(x - v.x, y - v.y);
    }
    
    inline Vector operator*(const Vector &v) const {
      return Vector(x * v.x, y * v.y);
    }

    inline Vector operator*(typ d) const {
      return Vector(x*d, y*d);
    }

    inline Vector operator/(typ d) const {
      return Vector(x / d, y / d);
    }

    inline void operator+=(const Vector &v) {
      x += v.x; y += v.y;
    }

    inline void operator-=(const Vector &v) {
      x -= v.x; y -= v.y;
    }

    inline void operator*=(const Vector &v) {
      x *= v.x; y *= v.y;
    }

    inline void operator*=(typ d) {
      x *= d; y *= d;
    }

    inline void operator/=(typ d) {
      x /= d; y /= d;
    }

  };


  template<typename typ> struct Vector<3, typ> {

    public:

      typ x, y, z;

      Vector() {
        x = y = z = 0;
      }

      Vector(typ v) {
        x = y = z = v;
      }

      Vector(typ X, typ Y) {
        x = X; y = Y; z = 0;
      }

      Vector(typ X, typ Y, typ Z) {
        x = X; y = Y; z = Z;
      }

      Vector(typ* v) {
        x = v[0]; y = v[1]; z = v[2];
      }

      template<typename typ_> Vector<3, typ>& operator = (const Vector<2, typ_>& other) {
        x = (typ)other.x;
        y = (typ)other.y;
        z = 0;
        return *this;
      }

      template<int dim_, typename typ_> operator Vector<dim_, typ_>() {
        return Vector<dim_, typ_>(static_cast<typ_>(x), static_cast<typ_>(y), static_cast<typ_>(z));
      }

      inline typ& operator[](unsigned int id) {
        return (&x)[id];
      }

      inline typ dot(const Vector &v) const {
        return x*v.x + y*v.y + z*v.z;
      }

      inline Vector cross(const Vector &v) const {
        return Vector(y * v.z - z * v.y, z * v.x - x * v.z, x * v.y - y * v.x);
      }

      inline Vector normalize() const {
        return Vector(x, y, z) / length();
      }

      inline typ lengthSq() const {
        return x*x + y*y + z*z;
      }

      inline typ length() const {
        return sqrt(lengthSq());
      }

      inline const Vector operator-(void) const {
        return Vector(-x, -y, -z);
      }

      inline Vector operator+(const Vector &v) const {
        return Vector(x + v.x, y + v.y, z + v.z);
      }

      inline Vector operator-(const Vector &v) const {
        return Vector(x - v.x, y - v.y, z - v.z);
      }

      inline Vector operator*(const Vector &v) const {
        return Vector(x * v.x, y * v.y, z * v.z);
      }

      inline Vector operator*(typ d) const {
        return Vector(x*d, y*d, z*d);
      }

      inline Vector operator/(typ d) const {
        return Vector(x / d, y / d, z / d);
      }

      inline void operator+=(const Vector &v) {
        x += v.x; y += v.y; z += v.z;
      }

      inline void operator-=(const Vector &v) {
        x -= v.x; y -= v.y; z -= v.z;
      }

      inline void operator*=(const Vector &v) {
        x *= v.x; y *= v.y; z *= v.z;
      }

      inline void operator*=(typ d) {
        x *= d; y *= d; z *= d;
      }

      inline void operator/=(typ d) {
        x /= d; y /= d; z /= d;
      }

  };


  template<typename typ> struct Vector<4, typ> {

    public:

      typ x, y, z, w;

      Vector() {
        x = y = z = w = 0;
      }

      Vector(typ v) {
        x = y = z = w = v;
      }

      Vector(typ X, typ Y, typ Z) {
        x = X; y = Y; z = Z; w = 0;
      }

      Vector(typ X, typ Y, typ Z, typ W) {
        x = X; y = Y; z = Z; w = W;
      }

      Vector(typ* v) {
        x = v[0]; y = v[1]; z = v[2]; w = v[3];
      }

      inline typ& operator [] (unsigned int id) {
        return (&x)[id];
      }

      inline typ dot(const Vector &v) {
        return x*v.x + y*v.y + z*v.z - w*v.w;
      }

      inline typ lengthSq() {
        return x*x + y*y + z*z + w*w;
      }

      inline typ length() {
        return sqrt(lengthSq());
      }

      inline const Vector operator-(void) const {
        return Vector(-x, -y, -z, -w);
      }

      inline Vector operator+(const Vector &v) const {
        return Vector(x + v.x, y + v.y, z + v.z, w + v.w);
      }

      inline Vector operator-(const Vector &v) const {
        return Vector(x - v.x, y - v.y, z - v.z, w - v.w);
      }

      inline Vector operator*(const Vector &v) const {
        return Vector(x * v.x, y * v.y, z * v.z, w * v.w);
      }

      inline Vector operator*(typ d) const {
        return Vector(x*d, y*d, z*d, w*d);
      }

      inline Vector operator/(typ d) const {
        return Vector(x / d, y / d, z / d, w / d);
      }

      inline void operator+=(const Vector &v) {
        x += v.x; y += v.y; z += v.z; w += v.w;
      }

      inline void operator-=(const Vector &v) {
        x -= v.x; y -= v.y; z -= v.z; w -= v.w;
      }

      inline void operator*=(const Vector &v) {
        x *= v.x; y *= v.y; z *= v.z; w *= v.w;
      }

      inline void operator*=(typ d) {
        x *= d; y *= d; z *= d; w *= d;
      }

      inline void operator/=(typ d) {
        x /= d; y /= d; z /= d; w /= d;
      }

  };


  template<int dim, typename typ, typename typ2> inline Vector<dim, typ> operator * (typ2 d, const Vector<dim, typ>& v) {
    return v * static_cast<typ>(d);
  }


  template<typename typ1, typename typ2> inline Vector<2, float> Vector2f(typ1 x, typ2 y) {
    return Vector<2, float>((float)x, (float)y);
  }


  template<typename typ1, typename typ2> inline Vector<2, double> Vector2d(typ1 x, typ2 y) {
    return Vector<2, double>(x, y);
  }


  template<typename typ1, typename typ2, typename typ3> inline Vector<3, float> Vector3f(typ1 x, typ2 y, typ3 z) {
    return Vector<3, float>((float)x, (float)y, (float)z);
  }


  template<typename typ1, typename typ2, typename typ3> inline Vector<3, double> Vector3d(typ1 x, typ2 y, typ3 z) {
    return Vector<3, double>(x, y, z);
  }

}