
__kernel void computeForce
(
  __global float3* pos, 
  __global float3* vel, 
  __global float3* acc, 
  __global float3* vorticity, 
  __global float* density, 
  __global float* isoDensity, 
  __global float* pressure, 
  const int usePressure,
  const int useViscosity,
  const int useVorticity,
  __global int* neighboursFluidFluid,
  const int neighboursFluidFluidCountMax,
  __global float3* solidPos, 
  __global float* solidVolume, 
  __global int* neighboursFluidSolid,
  const int neighboursFluidSolidCountMax
) 
{
  int i = get_global_id(0);
  int fluidStart = i*neighboursFluidFluidCountMax;
  int fluidEnd = fluidStart + neighboursFluidFluidCountMax;
  int solidStart = i*neighboursFluidSolidCountMax;
  int solidEnd = solidStart + neighboursFluidSolidCountMax;
  
  float3 posi = pos[i];
  float3 veli = vel[i];
  
  int j;
  
  float r, r2, rh, t, t2, densityij, tp, viscosityTerm, tensionTerm, vis;
  float3 vec, w;
  
  float densityi = density[i];
  float pressurei = pressure[i];  
  float3 vorticityi = vorticity[i];
  
  float v = solidViscosity0 * radius * 1.0 / (2 * densityi);
  float vddw = viscosity0 * ddWViscosity;
  float mass02 = 2*mass0;
  float pddw = -pressurei / (densityi * densityi) * dWPressure;
  float radius2001 = 0.01 * radius2;
  
  float3 accTmp = acc[i];
  
  accTmp.z += gravity;
  
  // Fluid-fluid loop
  for (int k = fluidStart; k < fluidEnd; k++) {
    j = neighboursFluidFluid[k];
    
    if (j > -1) {
      vec = posi - pos[j];
    
      r = length(vec); 
			t = 1 - r * invRadius;
      t2 = t*t;

      densityij = densityi * density[j];
      
			// PRESSURE
      if (usePressure) {
        tp = mass0 * (-0.5 * (pressurei + pressure[j]) / densityij * dWPressure * t2 / (r + 0.0001));
        accTmp += tp * vec;
      }
      
      // VISCOSITY
      if (useViscosity) {
        viscosityTerm = mass0 * (vddw * t / densityij);
        accTmp += viscosityTerm * (vel[j] - veli);
		  }
      
      // VORTICITY
      if (useVorticity) {
        w = mass0 * vorticityi + mass0 * vorticity[j];
        w = (float)(vorticityFactor / (r * mass02)) * cross(vec, w);
        accTmp -= w;
      }
      
      // TENSION OLD
      /*rh = 1 - (r2 / radius2);
      tensionTerm = surfaceTension * dWDensity * rh * rh;
      acc[i] += mass0 * tensionTerm * vec;*/
    }
    else {
      break;
    }
  }
  
  // Fluid-solid loop
  for (int k = solidStart; k < solidEnd; k++) {
    j = neighboursFluidSolid[k];
    
    if (j > -1) {
      vec = posi - solidPos[j];
    
      r2 = length2v(vec); 

		  r = sqrt(r2);
			t = 1 - r * invRadius;
      t2 = t*t;

			// PRESSURE
      if (usePressure) {
        tp = solidVolume[j] * (pddw * t2 / r);
        accTmp += tp * vec;
      }
      
      // VISCOSITY
      if (useViscosity) {
				vis = dot(veli, vec);
				if (vis >= 0) {
					vis = solidVolume[j] * (v * vis / (r2 + radius2001)) * dWPressure * t2 / (r + 0.0001);
          accTmp += vis * vec;
				}
      }
    }
    else {
      break;
    }
  }

  // Final sum
  acc[i] = accTmp;
}
