
__kernel void normalizeDensityFirstPass
(
  __global float3* pos,  
  __global float* density, 
  __global float* isoDensity, 
  __global int* neighboursFluidFluid,
  const int neighboursFluidFluidCountMax,
  __global float3* solidPos,  
  __global float* solidVolume, 
  __global int* neighboursFluidSolid,
  const int neighboursFluidSolidCountMax
) 
{
  int i = get_global_id(0);
  int fluidStart = i*neighboursFluidFluidCountMax;
  int fluidEnd = fluidStart + neighboursFluidFluidCountMax;
  int solidStart = i*neighboursFluidSolidCountMax;
  int solidEnd = solidStart + neighboursFluidSolidCountMax;
  
  float3 posi = pos[i];
  
  float isoDensityTmp = isoDensity[i];
  
  int j;
  
  float r2, t, t3;
  
  // Fluid-fluid loop
  for (int k = fluidStart; k < fluidEnd; k++) {
    j = neighboursFluidFluid[k];
    
    if (j > -1) {
      r2 = distance2v(posi, pos[j]); 

			t = 1 - r2 * invRadius2;
			t3 = t*t*t;

      // ISO DENSITY
		  isoDensityTmp += mass0 / density[j] * t3;
    }
    else {
      break;
    }
  }
  
  // Fluid-solid loop
  for (int k = solidStart; k < solidEnd; k++) {
    j = neighboursFluidSolid[k];
    
    if (j > -1) {
      r2 = distance2v(posi, solidPos[j]); 

			t = 1 - r2 * invRadius2;
			t3 = t*t*t;

      // ISO DENSITY
		  isoDensityTmp += solidVolume[j] / density0 * t3;
    }
    else {
      break;
    }
  }
  
  // Final sum
  isoDensity[i] = isoDensityTmp;
}
