
__kernel void predictDensity
(
  __global float3* pos, 
  __global float* density,  
  __global float* densityPrediction,  
  __global float3* velocityPrediction,
  __global float3* d_ii, 
  __global float* a_ii, 
  __global int* neighboursFluidFluid,
  const int neighboursFluidFluidCountMax,
  __global float3* solidPos, 
  __global float* solidVolume, 
  __global int* neighboursFluidSolid,
  const int neighboursFluidSolidCountMax
) 
{
  int i = get_global_id(0);
  int fluidStart = i*neighboursFluidFluidCountMax;
  int fluidEnd = fluidStart + neighboursFluidFluidCountMax;
  int solidStart = i*neighboursFluidSolidCountMax;
  int solidEnd = solidStart + neighboursFluidSolidCountMax;
  
  float3 posi = pos[i];
  
  float densityi = density[i];
  float3 velocityPredictioni = velocityPrediction[i];
  float3 d_iii = d_ii[i];
  
  int j;
  
  float r, t, tp, dtp;
  float3 vec;
  float3 d_ji;
  float deltaTimeMassDensity = deltaTime2 * mass0 / (densityi * densityi);
  
  float densityPredictionTmp = densityPrediction[i];
  float a_iiTmp = a_ii[i];
  
  // Fluid-fluid loop
  for (int k = fluidStart; k < fluidEnd; k++) {
    j = neighboursFluidFluid[k];
    
    if (j > -1) {
      vec = posi - pos[j];
    
      r = length(vec); 
			t = 1 - r * invRadius;
      
      tp = dWPressure * t * t / r;
	    
			densityPredictionTmp += mass0 * (deltaTime * tp * dot(velocityPredictioni - velocityPrediction[j], vec));
      
      dtp = deltaTimeMassDensity * tp;
      d_ji = dtp * vec;

		  a_iiTmp += mass0 * tp * dot(d_iii - d_ji, vec);
    }
    else {
      break;
    }
  }
  
  // Fluid-solid loop
  for (int k = solidStart; k < solidEnd; k++) {
    j = neighboursFluidSolid[k];
    
    if (j > -1) {
      vec = posi - solidPos[j];
    
      r = length(vec); 
			t = 1 - r * invRadius;
      
      tp = dWPressure * t * t / r;
      
      densityPredictionTmp += solidVolume[j] * deltaTime * tp * dot(velocityPredictioni, vec);
   
      a_iiTmp += solidVolume[j] * tp * dot(d_iii, vec);
    }
    else {
      break;
    }
  }
  
  // Final sum
  densityPrediction[i] = densityPredictionTmp;
  a_ii[i] = a_iiTmp;
}
