
__kernel void applyXSPHFirstPass
(
  __global float3* pos,  
  __global float3* vel, 
  __global float3* velXSPH, 
  __global float* density, 
  __global int* neighboursFluidFluid,
  const int neighboursFluidFluidCountMax
) 
{
  int i = get_global_id(0);
  int fluidStart = i*neighboursFluidFluidCountMax;
  int fluidEnd = fluidStart + neighboursFluidFluidCountMax;
  
  float3 posi = pos[i];
  float3 veli = vel[i];

  int j;
  
  float r2, t, t2, t3;
  float3 vec;
  
  float3 tmp = (0, 0, 0);
  
  // Fluid-fluid loop
  for (int k = fluidStart; k < fluidEnd; k++) {
    j = neighboursFluidFluid[k];
    
    if (j > -1) {
      vec = posi - pos[j];
    
      r2 = length2v(vec); 

		  t = 1 - r2 * invRadius2;
      t2 = t*t;
			t3 = t2*t;

			tmp += t3 * (vel[j] - veli) / density[j];
    }
    else {
      break;
    }
  }
  
  
  // Final sum
  velXSPH[i] = tmp;
}
