

__kernel void findNeighboursFluid
(
  __global float3* pos,  
  __global ulong2* gridCellsFluid,
  __global int* gridCellIndicesFluid,
  __global ulong* lastCellFluid,
  __global int* neighboursFluidFluid,
  __global int* neighboursFluidFluidCount,
  const int neighboursFluidFluidCountMax,
  __global float3* solidPos, 
  __global ulong2* gridCellsSolid,
  __global int* gridCellIndicesSolid,
  __global ulong* lastCellSolid,
  __global int* neighboursFluidSolid,
  __global int* neighboursFluidSolidCount,
  const int neighboursFluidSolidCountMax  
) 
{
  int i = get_global_id(0);
  int offsetFluid = i*neighboursFluidFluidCountMax;
  int offsetSolid = i*neighboursFluidSolidCountMax;
  
  float3 posi = pos[i];
  
  int posXic = posi.x * invRadius;  
  int posYic = posi.y * invRadius;
  int posZic = posi.z * invRadius;
  
  int kf = 0, ks = 0;
  int x, y, z, cx, cy, cz, j, jj;
  ulong zOrderPos;
  
  ulong lastCellSolid0 = lastCellSolid[0];
  ulong lastCellFluid0 = lastCellFluid[0];
  
  for (x = -1; x <= 1; x++) {
    for (y = -1; y <= 1; y++) {
      for (z = -1; z <= 1; z++) {
        cx = posXic + x; cy = posYic + y; cz = posZic + z;
        
        if (cx >= 0 && cy >= 0 && cz >= 0) { 
          zOrderPos = zOrder(cx, cy, cz); 
        
          if (zOrderPos <= lastCellFluid0) {
            for (j = gridCellIndicesFluid[zOrderPos]; gridCellsFluid[j].x == zOrderPos; j++) {
              jj = gridCellsFluid[j].y;
              if (jj != i && distance2(posi.x, posi.y, posi.z, pos[jj].x, pos[jj].y, pos[jj].z) < radius2) {
                if (kf < neighboursFluidFluidCountMax) {
                  neighboursFluidFluid[offsetFluid + kf] = jj;
                }
                kf++;
              }
            }
          }
          
          if (zOrderPos <= lastCellSolid0) {
            for (j = gridCellIndicesSolid[zOrderPos]; gridCellsSolid[j].x == zOrderPos; j++) {
              jj = gridCellsSolid[j].y;
              if (jj != i && distance2(posi.x, posi.y, posi.z, solidPos[jj].x, solidPos[jj].y, solidPos[jj].z) < radius2) {
                if (ks < neighboursFluidSolidCountMax) {
                  neighboursFluidSolid[offsetSolid + ks] = jj;
                }
                ks++;
              }
            }
          }
        }
      }
    }
  }
  
  if (kf < neighboursFluidFluidCountMax) {
    neighboursFluidFluid[offsetFluid + kf] = -1;
  }
  neighboursFluidFluidCount[i] = kf;
  
  
  if (ks < neighboursFluidSolidCountMax) {
    neighboursFluidSolid[offsetSolid + ks] = -1;
  }
  neighboursFluidSolidCount[i] = ks;
}
