
__kernel void findNeighboursSolid
(
  __global float3* solidPos, 
  __global ulong2* gridCellsSolid,
  __global int* gridCellIndicesSolid,
  __global ulong* lastCellSolid,
  __global int* neighboursSolidSolid,
  __global int* neighboursSolidSolidCount,
  const int neighboursSolidSolidCountMax  
) 
{
  int i = get_global_id(0);
  int offsetSolid = i*neighboursSolidSolidCountMax;
  
  float3 solidPosi = solidPos[i];
  
  int solidPosXic = solidPosi.x * invRadius;
  int solidPosYic = solidPosi.y * invRadius;
  int solidPosZic = solidPosi.z * invRadius;
  
  int ks = 0;
  int x, y, z, cx, cy, cz, j, jj;
  ulong zOrderPos;
  
  ulong lastCellSolid0 = lastCellSolid[0];
  
  for (x = -1; x <= 1; x++) {
    for (y = -1; y <= 1; y++) {
      for (z = -1; z <= 1; z++) {
        cx = solidPosXic + x; cy = solidPosYic + y; cz = solidPosZic + z;
      
        if (cx >= 0 && cy >= 0 && cz >= 0) { 
          zOrderPos = zOrder(cx, cy, cz); 
          
          if (zOrderPos <= lastCellSolid0) {
            for (j = gridCellIndicesSolid[zOrderPos]; gridCellsSolid[j].x == zOrderPos; j++) {
              jj = gridCellsSolid[j].y;
              if (jj != i && distance2(solidPosi.x, solidPosi.y, solidPosi.z, solidPos[jj].x, solidPos[jj].y, solidPos[jj].z) < radius2) {
                if (ks < neighboursSolidSolidCountMax) {
                  neighboursSolidSolid[offsetSolid + ks] = jj;
                }
                ks++;
              }
            }
          }
        }
      }
    }
  }
  
  if (ks < neighboursSolidSolidCountMax) {
    neighboursSolidSolid[offsetSolid + ks] = -1;
  }
  neighboursSolidSolidCount[i] = ks;
}
