#pragma once;

class OpenGL;

class Camera {

  protected:

    glm::vec3 position;
    glm::vec3 lookAt;
    glm::vec3 up;

    OpenGL* opengl;

    Camera(OpenGL* ogl);
    Camera(OpenGL* ogl, glm::vec3& position, glm::vec3& lookAt, glm::vec3& up);
    Camera(Camera const&);  

    void operator=(Camera const&);

  public:

    virtual GLvoid useCamera();

    virtual GLvoid keyDown(GLint key) = 0;
    virtual GLvoid keyUp(GLint key) = 0;

    virtual GLvoid mouseDown(GLint mouseButton, GLint x, GLint y) = 0;
    virtual GLvoid mouseUp(GLint mouseButton, GLint x, GLint y) = 0;
    virtual GLvoid mouseMove(GLint mouseButton, GLint x, GLint y) = 0;
    virtual GLvoid mouseScroll(GLfloat delta) = 0;

    glm::mat4 getViewMatrix();

    glm::vec3 getPosition()                                                                                 { return position; };
    glm::vec3 getLookAt()                                                                                   { return lookAt; };
    glm::vec3 getUp()                                                                                       { return up; };

};

class OrbitCamera: public Camera {

  private:

    GLfloat azimuth, altitude;
    GLfloat azimuthOld, altitudeOld;
    GLfloat distance;

    glm::vec2 lastMouseMovePos;
    GLboolean wasMouseMove;

    GLboolean freeCamera;
    GLfloat moveSpeed;

    GLvoid computePosition();
    GLvoid moveCamera();

  public:

    OrbitCamera(OpenGL* ogl, GLfloat azimuth, GLfloat altitude, GLfloat distance);
    OrbitCamera(OpenGL* ogl, GLfloat azimuth, GLfloat altitude, glm::vec3 position, GLfloat moveSpeed);
    
    GLvoid useCamera();

    GLvoid keyDown(GLint key);
    GLvoid keyUp(GLint key);

    GLvoid mouseDown(GLint mouseButton, GLint x, GLint y);
    GLvoid mouseUp(GLint mouseButton, GLint x, GLint y);
    GLvoid mouseMove(GLint mouseButton, GLint x, GLint y);
    GLvoid mouseScroll(GLfloat delta);

};

class FreeCamera: public Camera {

  private:

    glm::vec2 screenPosition;

    GLfloat moveSpeed;
    GLfloat sensitivity;	

    GLvoid rotateCamera();
    GLvoid moveCamera();

  public:

    FreeCamera(OpenGL* ogl, glm::vec3& position, glm::vec3& lookDirection, glm::vec3& up, GLfloat moveSpeed, GLfloat sensitivity);

    GLvoid useCamera();

    GLvoid keyDown(GLint key);
    GLvoid keyUp(GLint key);

    GLvoid mouseDown(GLint mouseButton, GLint x, GLint y);
    GLvoid mouseUp(GLint mouseButton, GLint x, GLint y);
    GLvoid mouseMove(GLint mouseButton, GLint x, GLint y);
    GLvoid mouseScroll(GLfloat delta);

};