#include "../main.h"

CamerasManager::CamerasManager(OpenGL* ogl): 
  opengl(ogl),
  currentCamera(-1) {

}

CamerasManager::~CamerasManager() {

}

GLboolean CamerasManager::initCameras() {
  OrbitCamera* orbitCamera = new OrbitCamera(opengl, 0.7f, 0.63f, 1.0f);
  cameras.push_back(orbitCamera);

  glm::vec3 position(0.5, 0.5, 0.5);
  glm::vec3 lookDirection(-1.0, -1.0, -1.0);
  lookDirection = glm::normalize(lookDirection);
  glm::vec3 up(0.0, 0.0, 1.0);
  
  orbitCamera = new OrbitCamera(opengl, 0.77f, 0.61f, position, 0.01f);
  cameras.push_back(orbitCamera);

  FreeCamera* freeCamera = new FreeCamera(opengl, position, lookDirection, up, 0.01f, 1.0f);
  cameras.push_back(freeCamera);

  currentCamera = Settings::getInstance().getCamera();
  
  return true;
}

GLvoid CamerasManager::freeCameras() {
  for (int i = 0; i < cameras.size(); i++) {
    delete cameras[i];
  }
  cameras.clear();
  currentCamera = -1;
}

GLvoid CamerasManager::keyDown(GLint key) {
  cameras[currentCamera]->keyDown(key);
}

GLvoid CamerasManager::keyUp(GLint key) {
  switch (key) {
    case 'C': {
      currentCamera = (currentCamera + 1) % 3;
      break;
    }
  }

  cameras[currentCamera]->keyUp(key);
}

GLvoid CamerasManager::mouseDown(GLint mouseButton, GLint x, GLint y) { 
  cameras[currentCamera]->mouseDown(mouseButton, x, y);
}

GLvoid CamerasManager::mouseUp(GLint mouseButton, GLint x, GLint y) { 
  cameras[currentCamera]->mouseUp(mouseButton, x, y);
}

GLvoid CamerasManager::mouseMove(GLint mouseButton, GLint x, GLint y) { 
  cameras[currentCamera]->mouseMove(mouseButton, x, y);
}

GLvoid CamerasManager::mouseScroll(GLfloat delta) { 
  cameras[currentCamera]->mouseScroll(delta);
}