#include "../main.h"

LightsManager::LightsManager() {

}

LightsManager::~LightsManager() {

}

GLint LightsManager::numEnabledLights() {
  int num = 0;
  
  for (int i = 0; i < lights.size(); i++) {
    if (lights[i]->isEnabled()) {
      num++;
    }
  }

  return num;
}

GLboolean LightsManager::initLights() {
  glm::vec4 lightAmbient0(0.3, 0.3, 0.3, 1.0);
  glm::vec4 lightDiffuse0(1.0, 1.0, 1.0, 1.0);
  glm::vec4 lightSpecular0(1.0, 1.0, 1.0, 1.0);
  glm::vec4 lightPosition0(400.0, 400.0, 400.0, 1.0);

  glm::vec4 lightAmbient1(0.1, 0.1, 0.1, 1.0);
  glm::vec4 lightDiffuse1(0.5, 0.5, 0.5, 1.0);
  glm::vec4 lightSpecular1(1.0, 1.0, 1.0, 1.0);
  glm::vec4 lightPosition1(400.0, -400.0, 400.0, 1.0);

  glm::vec4 lightAmbient2(0.1, 0.1, 0.1, 1.0);
  glm::vec4 lightDiffuse2(0.5, 0.5, 0.5, 1.0);
  glm::vec4 lightSpecular2(1.0, 1.0, 1.0, 1.0);
  glm::vec4 lightPosition2(-400.0, 400.0, 400.0, 1.0);

  glm::vec4 lightAmbient3(0.1, 0.1, 0.1, 1.0);
  glm::vec4 lightDiffuse3(0.5, 0.5, 0.5, 1.0);
  glm::vec4 lightSpecular3(1.0, 1.0, 1.0, 1.0);
  glm::vec4 lightPosition3(-400.0, -400.0, 400.0, 1.0);

  Light* light0 = new Light(lightAmbient0, lightDiffuse0, lightSpecular0, lightPosition0);
  light0->enableLight();

  /*Light* light1 = new Light(lightAmbient1, lightDiffuse1, lightSpecular1, lightPosition1);
  light1->enableLight();

  Light* light2 = new Light(lightAmbient2, lightDiffuse2, lightSpecular2, lightPosition2);
  light2->enableLight();

  Light* light3 = new Light(lightAmbient3, lightDiffuse3, lightSpecular3, lightPosition3);
  light3->enableLight();*/
  
  lights.push_back(light0);
  /*lights.push_back(light1);
  lights.push_back(light2);
  lights.push_back(light3);*/

  return true;
}

GLvoid LightsManager::freeLights() {
  for (int i = 0; i < lights.size(); i++) {
    delete lights[i];
  }
  lights.clear();
}

GLvoid LightsManager::applyEnabledLights(Camera* camera) {
  RenderOptions* renderOptions = &RenderOptions::getInstance();

  glm::mat4 viewMatrix = camera->getViewMatrix();

  const GLint enabledLights = numEnabledLights();

  glm::vec4* ambient = new glm::vec4[enabledLights];
  glm::vec4* diffuse = new glm::vec4[enabledLights];
  glm::vec4* specular = new glm::vec4[enabledLights];
  glm::vec4* position = new glm::vec4[enabledLights];

  int current = 0;
  for (int i = 0; i < lights.size(); i++) {
    if (lights[i]->isEnabled()) {
      ambient[current] = lights[i]->getAmbient();
      diffuse[current] = lights[i]->getDiffuse();
      specular[current] = lights[i]->getSpecular();
      position[current] = viewMatrix * lights[i]->getPosition();
      current++;
    }
  }

  GLint shLightAmbient = glGetUniformLocation(renderOptions->getGLSLprogram(), "lightAmbient");
  glUniform4fv(shLightAmbient, enabledLights, &(ambient)[0][0]);

  GLint shLightDiffuse = glGetUniformLocation(renderOptions->getGLSLprogram(), "lightDiffuse");
  glUniform4fv(shLightDiffuse, enabledLights, &(diffuse)[0][0]);

  GLint shLightSpecular = glGetUniformLocation(renderOptions->getGLSLprogram(), "lightSpecular");
  glUniform4fv(shLightSpecular, enabledLights, &(specular)[0][0]);

  GLint shLightPosition = glGetUniformLocation(renderOptions->getGLSLprogram(), "lightPosition");
  glUniform4fv(shLightPosition, enabledLights, &(position)[0][0]);
}

GLvoid LightsManager::cancelEnabledLights() {
  RenderOptions* renderOptions = &RenderOptions::getInstance();

  GLint shLightAmbient = glGetUniformLocation(renderOptions->getGLSLprogram(), "lightAmbient");
  glUniform4fv(shLightAmbient, 0, 0);

  GLint shLightDiffuse = glGetUniformLocation(renderOptions->getGLSLprogram(), "lightDiffuse");
  glUniform4fv(shLightDiffuse, 0, 0);

  GLint shLightSpecular = glGetUniformLocation(renderOptions->getGLSLprogram(), "lightSpecular");
  glUniform4fv(shLightSpecular, 0, 0);

  GLint shLightPosition = glGetUniformLocation(renderOptions->getGLSLprogram(), "lightPosition");
  glUniform4fv(shLightPosition, 0, 0);
}

GLvoid LightsManager::renderEnabledLights(Camera* camera) {
  for (int i = 0; i < lights.size(); i++) {
    if (lights[i]->isEnabled()) {
      lights[i]->renderLight(camera);
    }
  }
}

GLvoid LightsManager::keyDown(GLint key) {

}

GLvoid LightsManager::keyUp(GLint key) {

}

GLvoid LightsManager::mouseDown(GLint mouseButton, GLint x, GLint y) {

}

GLvoid LightsManager::mouseUp(GLint mouseButton, GLint x, GLint y) {

}

GLvoid LightsManager::mouseMove(GLint mouseButton, GLint x, GLint y) {

}

GLvoid LightsManager::mouseScroll(GLfloat delta) {

}