#include "../main.h"

Material::Material() {
  
}

Material::~Material() {
  deleteMaterial();
}

GLvoid Material::deleteMaterial() {

}

GLboolean Material::loadMaterial(glm::vec4 ambient, glm::vec4 diffuse, glm::vec4 specular, glm::vec4 emission, GLfloat shininess, GLenum polygonMode, GLboolean faceCulling) {
  this->ambient = ambient;
  this->diffuse = diffuse;
  this->specular = specular;
  this->emission = emission;

  this->shininess = shininess;

  this->faceCulling = faceCulling;
  
  return true;
}

GLvoid Material::applyMaterial() {
  Settings* settings = &Settings::getInstance();
  RenderOptions* renderOptions = &RenderOptions::getInstance();

  glColor3f(diffuse.r, diffuse.g, diffuse.b);
  
  GLint shMaterialAmbient = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialAmbient");
  glUniform4fv(shMaterialAmbient, 1, &ambient[0]);

  GLint shMaterialDiffuse = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialDiffuse");
  glUniform4fv(shMaterialDiffuse, 1, &diffuse[0]);

  GLint shMaterialSpecular = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialSpecular");
  glUniform4fv(shMaterialSpecular, 1, &specular[0]);

  GLint shMaterialEmission = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialEmission");
  glUniform4fv(shMaterialEmission, 1, &emission[0]);

  GLint shMaterialShininess = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialShininess");
  glUniform1f(shMaterialShininess, shininess);

  //applyMode();
}

GLvoid Material::applyMode() {
  RenderOptions* renderOptions = &RenderOptions::getInstance();

  if (!renderOptions->getForceWireframe()) {
    glPolygonMode(GL_FRONT_AND_BACK, polygonMode);

    if (faceCulling) {
      glEnable(GL_CULL_FACE);
    }
    else {
      glDisable(GL_CULL_FACE);
    }
  }
  else {
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    glEnable(GL_CULL_FACE);
  }
}

GLvoid Material::cancelMaterial() {
  RenderOptions* renderOptions = &RenderOptions::getInstance();

  glm::vec4 def(1.0, 1.0, 1.0, 1.0);

  GLint shMaterialAmbient = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialAmbient");
  glUniform4fv(shMaterialAmbient, 1, &def[0]);

  GLint shMaterialDiffuse = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialDiffuse");
  glUniform4fv(shMaterialDiffuse, 1, &def[0]);

  GLint shMaterialSpecular = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialSpecular");
  glUniform4fv(shMaterialSpecular, 1, &def[0]);

  GLint shMaterialEmission = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialEmission");
  glUniform4fv(shMaterialEmission, 1, &def[0]);

  GLint shMaterialShininess = glGetUniformLocation(renderOptions->getGLSLprogram(), "materialShininess");
  glUniform1f(shMaterialShininess, 0.0f);
}