#pragma once

class Window;

class OpenGL {

  private:

    GLint lastTicks;
    GLint fps;
    GLint frames;

    GLuint fpsFont2D;
    GLuint infoFont2D;

    GLfloat zNear;
    GLfloat zFar;

    Material* material;
    GLUquadricObj* quadric;

    ShadersManager* shadersManager;
    CamerasManager* camerasManager;
    LightsManager* lightsManager;

    Window* window;

    GLvoid createFont2D(GLuint &font2D, std::string fontName, GLint fontSize, GLboolean fontBold = true);
    GLvoid deleteFont2D(GLuint &font2D);

    GLvoid countFPS();
    GLvoid glPrint2D(GLuint font, const GLchar *fmt, ...);
    GLvoid glPrint2D(GLuint font, const std::string& str);

    GLvoid renderScene(Scene* scene);
    GLvoid renderAxes(GLfloat scale);
    GLvoid renderFPS();
    GLvoid renderInfo(Scene* scene);

  public:

    OpenGL(Window* wd);
    ~OpenGL();                               

    GLboolean initOpenGL();
    GLvoid freeOpenGL();

    GLvoid createFonts();
    GLvoid deleteFonts();

    GLvoid updateSize();

    GLvoid renderInitInfo(std::string str);
    GLvoid render(Scene* scene);
    
    GLvoid keyDown(GLint key);
    GLvoid keyUp(GLint key);

    GLvoid mouseDown(GLint mouseButton, GLint x, GLint y);
    GLvoid mouseUp(GLint mouseButton, GLint x, GLint y);
    GLvoid mouseMove(GLint mouseButton, GLint x, GLint y);
    GLvoid mouseScroll(GLfloat delta);

    GLfloat getZNear() { return zNear; };
    GLfloat getZFar() { return zFar; };
    GLhandleARB getProgram(std::string programName);
    Window* getWindow() { return window; };

    static GLvoid applyBillboard(Camera* camera, glm::vec3 position);
    static glm::vec3 OpenGL::getMouseClick(Camera* camera, glm::vec2 cursor, GLboolean withDepth = true);

    static GLdouble angleCos(glm::vec3 a, glm::vec3 b);

    static float length2(float vecX, float vecY, float vecZ);

};