#include "main.h"

Log* Log::logInstance = NULL;

Log::Log(): 
  fileName("oclsph_log.txt"), 
  file(fileName),
  newLine(true) { 

  file << "==================" << std::endl << "    oclSPH log" << std::endl << "==================" << std::endl; 
  file.close();
}

Log::~Log() { 
  if (file.is_open()) {
    file.close();
  }

  supress.clear();

  logInstance = NULL;
}

std::string Log::logTypeString(LogType type) {
  switch (type) {
    case CLDebug:      return "Debug cl";
    case CLError:      return "ERROR cl";
    case CLInfo:       return "Info cl";
    case GLDebug:      return "Debug gl";
    case GLError:      return "ERROR gl";
    case GLInfo:       return "Info gl";
    case WndDebug:     return "Debug wnd";
    case WndError:     return "ERROR wnd";
    case WndInfo:      return "Info wnd";
    case SPHDebug:     return "Debug sph";
    case SPHError:     return "ERROR sph";
    case SPHInfo:      return "Info sph";
    case Custom:       return "Custom";
    default:           return "Unknown";
  }
}

std::string Log::deleteNullChars(std::string str) {
  str.erase(std::remove(str.begin(), str.end(), '\0'), str.end());
  return str;
}

void Log::writeLog(LogType type, const std::string& str) {
  file.open(fileName, std::ios::app);
  file << str;
  file.close();

  std::cout << str;
}

void Log::write(LogType type, const std::string& str) {
  if (supress.find(type) != supress.end()) {
    return;
  }

  std::ostringstream ss;
  if (newLine && str != "") {
    ss << ":: ";
    newLine = false;
  }
  ss << Log::logTypeString(type) << ": " << Log::deleteNullChars(str);

  writeLog(type, ss.str());
}

void Log::writeLine(LogType type, const std::string& str) {
  if (supress.find(type) != supress.end()) {
    return;
  }

  std::ostringstream ss;
  if (newLine && str != "") {
    ss << ":: ";
  }
  ss << Log::logTypeString(type) << ": " << Log::deleteNullChars(str) << std::endl;

  writeLog(type, ss.str());

  newLine = true;
}

void Log::write(const std::string& str) {
  write(LogType::Custom, str);
}

void Log::writeLine(const std::string& str) {
  writeLine(LogType::Custom, str);
}

void Log::supressLogType(LogType type) {
  supress.insert(type);
}

Log* Log::getInstance() { 
  if (logInstance == NULL) { 
    logInstance = new Log(); 
  } 

  return logInstance; 
}