#include "main.h"

std::vector<std::string> Settings::splitString(std::string str, char delim) {
  std::vector<std::string> result;
  int last = 0;
  std::string part;

  for (int i = 0; i < str.length(); i++) {
    if (str[i] == delim/* || i == str.length()-1*/) {
      part = str.substr(last, i-last);
      if (part != "") {
        result.push_back(part);
      }
      last = i+1;
    }
  }

  return result;
}

std::string Settings::trim(std::string str) {
  std::ostringstream ss;

  bool ok = false;
  for (int i = 0; i < str.length(); i++) {
    if (str.at(i) != ' ' || ok) {
      ss << str.at(i);
      ok = true;
    }
  }

  str = ss.str();
  ss.str("");

  ok = false;
  std::string tmp;
  for (int i = str.length()-1; i >= 0; i--) {
    if (str.at(i) != ' ' || ok) {
      tmp = ss.str();
      ss.str("");
      ss << str.at(i) << tmp;
      ok = true;
    }
  }

  return ss.str();
}

std::string Settings::getLineKey(std::string line) {
  std::string result;

  int i = -1;
  if ((i = line.find("=")) >= 0) {
    result = line.substr(0, i-1);
    result = trim(result);
  }
  else {
    throw 1;
  }

  return result;
}

std::string Settings::getLineValue(std::string line) {
  std::string result;

  int i = -1;
  if ((i = line.find("=")) >= 0) {
    result = line.substr(i+1);
    result = trim(result);

    if (result.at(result.length()-1) == ';') {
      result = result.substr(0, result.length()-1);
      result = trim(result);
    }
  }
  else {
    throw 1;
  }

  return result;
}

Settings& Settings::getInstance() {
  static Settings instance; 
  return instance;
}

void Settings::loadSettings(std::string fileName) {
  if (fileName != "") {
    std::ifstream file;

    try {
      file.open(fileName.c_str(), std::ios::in);
    
      if (file.is_open()) {
        std::string line;
        int tmp;

        while (file.good()) {
          std::getline(file, line);
          line = trim(line);
          std::vector<std::string> strings = splitString(line, ';');

          for (int i = 0; i < strings.size(); i++) {
            std::string key = getLineKey(strings[i]);
            std::string value = getLineValue(strings[i]);

            if (key == "width") {
              if ((tmp = atoi(value.c_str())) >= 200) {
                width = tmp;
              }
            } 
            else if (key == "height") {
              if ((tmp = atoi(value.c_str())) >= 150) {
                height = tmp;
              }
            } 
            else if (key == "color_depth") {
              if ((tmp = atoi(value.c_str())) == 16 || tmp == 32) {
                colorDepth = tmp;
              }
            } 
            else if (key == "msaa") {
              if ((tmp = atoi(value.c_str())) == 0 || tmp == 1 || tmp == 2 || tmp == 4 || tmp == 8 || tmp == 16) {
                msaa = tmp;
              }
            }
            else if (key == "maximized") {
              maximized = atoi(value.c_str());
            } 
            else if (key == "fullscreen") {
              fullscreen = atoi(value.c_str());
            } 
            else if (key == "camera") {
              if ((tmp = atoi(value.c_str())) >= 0 && tmp <= 2) {
                camera = tmp;
              }
            } 
            else if (key == "spheres") {
              spheres = atoi(value.c_str());
            }
            else if (key == "show_fps") {
              showFPS = atoi(value.c_str());
            } 
            else if (key == "show_info") {
              showInfo = atoi(value.c_str());
            }
            else if (key == "show_axes") {
              showAxes = atoi(value.c_str());
            }
            else if (key == "show_wireframe") {
              showWireframe = atoi(value.c_str());
            }
            else if (key == "show_lights") {
              showLights = atoi(value.c_str());
            }
            else if (key == "particles_visualization") {
              if ((tmp = atoi(value.c_str())) >= 0 && tmp <= 2) {
                fluidParticlesVisualization = tmp;
              }
            }
            else if (key == "show_solid_particles") {
              showSolidParticles = atoi(value.c_str());
            }
            else if (key == "device") {
              if ((tmp = atoi(value.c_str())) >= 0 && tmp <= 1) {
                currentDevice = tmp;
              }
            }
            else if (key == "simulation_method") {
              if ((tmp = atoi(value.c_str())) >= 0 && tmp <= 2) {
                simulationMethod = tmp;
              }
            }
            else if (key == "compute_grid_every_frame") {
              computeGridEveryFrame = atoi(value.c_str());
            }
            else if (key == "scene") {
              currentScene = atoi(value.c_str());
            }
            else if (key == "vortex") {
              vortex = atoi(value.c_str());
            }
            else if (key == "point_size") {
              if ((tmp = atoi(value.c_str())) >= 1 && tmp <= 15) {
                pointSize = tmp;
              }
            }
          }
        }
      }
    }
    catch (...) {
      if (file.is_open()) {
        file.close();
      }
    }
  
    if (file.is_open()) {
      file.close();
    }
  }
}

void Settings::saveSettings(std::string fileName) {
  if (fileName != "") {
    std::ofstream file;

    try {
      file.open(fileName.c_str(), std::ios::out);
    
      if (file.is_open()) {
        file << "width = " << width << ";" << std::endl;
        file << "height = " << height << ";" << std::endl;
        file << "color_depth = " << colorDepth << ";" << std::endl;
        file << "msaa = " << msaa << ";" << std::endl;
        file << "maximized = " << maximized << ";" << std::endl;
        file << "fullscreen = " << fullscreen << ";" << std::endl;
        file << "camera = " << camera << ";" << std::endl;
        file << "spheres = " << spheres << ";" << std::endl;
        file << "show_fps = " << showFPS << ";" << std::endl;
        file << "show_info = " << showInfo << ";" << std::endl;
        file << "show_axes = " << showAxes << ";" << std::endl;
        file << "show_wireframe = " << showWireframe << ";" << std::endl;
        file << "show_lights = " << showLights << ";" << std::endl;
        file << "particles_visualization = " << fluidParticlesVisualization << ";" << std::endl;
        file << "show_solid_particles = " << showSolidParticles << ";" << std::endl;
        file << "device = " << currentDevice << ";" << std::endl;
        file << "simulation_method = " << simulationMethod << ";" << std::endl;
        file << "compute_grid_every_frame = " << computeGridEveryFrame << ";" << std::endl;
        file << "scene = " << currentScene << ";" << std::endl;
        file << "vortex = " << vortex << ";" << std::endl;
        file << "point_size = " << pointSize << ";" << std::endl;
      }
    }
    catch (...) {
      if (file.is_open()) {
        file.close();
      }
    }
  
    if (file.is_open()) {
      file.close();
    }
  }
}