#pragma once

class Settings {

  private:

    int width, height;
    int colorDepth;
    int msaa;
    bool maximized;
    bool fullscreen;

    bool spheres;
    bool showFPS, showInfo;
    bool showAxes;
    bool showWireframe;
    bool showLights;
    int fluidParticlesVisualization;
    bool showSolidParticles;

    bool simulation;
    float simulationTime;

    int simulationMethod;
    bool computeGridEveryFrame;
    int currentScene;

    int currentDevice;
    int currentDeviceLocalSize;
    int buffersMemory;

    bool vortex;

    float pointSize;

    int camera;

    int mergeSort1Time;
    int mergeSort2Time;
    int computeGridTime;
    int neighboursTime;
    int reallocNeighboursBufferTime;
    int computeDensityTime;
    int computeForceTime;
    int integrateTime;
    int stateManagingTime;
    int overallComputationTime;
    int downloadTime;
    int renderingTime;

    Settings() : width(1024), height(768), colorDepth(32), msaa(0), maximized(false), fullscreen(false), spheres(false), showFPS(true), showInfo(true), showAxes(false), showWireframe(false), showLights(true), fluidParticlesVisualization(1), showSolidParticles(false), simulation(false), simulationTime(0.0f), simulationMethod(0), computeGridEveryFrame(true), currentScene(0), currentDevice(1), currentDeviceLocalSize(0), buffersMemory(0), vortex(false), pointSize(6.0f), camera(1), mergeSort1Time(0), mergeSort2Time(0), computeGridTime(0), neighboursTime(0), reallocNeighboursBufferTime(0), computeDensityTime(0), computeForceTime(0), integrateTime(0), stateManagingTime(0), overallComputationTime(0), downloadTime(0), renderingTime(0) {};
    Settings(Settings const&);                 
    
    void operator=(Settings const&); 
    std::vector<std::string> splitString(std::string str, char delim);
    std::string trim(std::string str);
    std::string getLineKey(std::string line);
    std::string getLineValue(std::string line);

  public:

    static Settings& getInstance();

    void loadSettings(std::string fileName);
    void saveSettings(std::string fileName);

    int getWidth()                                                                                          { return width; };
    void setWidth(int value)                                                                                { width = value; };

    int getHeight()                                                                                         { return height; };
    void setHeight(int value)                                                                               { height = value; };

    int getColorDepth()                                                                                     { return colorDepth; };
    void setColorDepth(int value)                                                                           { colorDepth = value; };

    int getMSAA()                                                                                           { return msaa; };
    void setMSAA(int value)                                                                                 { msaa = value; };

    bool getFullscreen()                                                                                    { return fullscreen; };
    void setFullscreen(bool value)                                                                          { fullscreen = value; };

    bool getMaximized()                                                                                     { return maximized; };
    void setMaximized(bool value)                                                                           { maximized = value; };

    bool getShowAxes()                                                                                      { return showAxes; };
    void setShowAxes(bool value)                                                                            { showAxes = value; };

    bool getShowWireframe()                                                                                 { return showWireframe; };
    void setShowWireframe(bool value)                                                                       { showWireframe = value; };

    bool getShowLights()                                                                                    { return showLights; };
    void setShowLights(bool value)                                                                          { showLights = value; };

    int getFluidParticlesVisualization()                                                                    { return fluidParticlesVisualization; };
    void setFluidParticlesVisualization(int value)                                                          { fluidParticlesVisualization = value; };

    bool getShowSolidParticles()                                                                            { return showSolidParticles; };
    void setShowSolidParticles(bool value)                                                                  { showSolidParticles = value; };

    bool getSpheres()                                                                                       { return spheres; };
    void setSpheres(bool value)                                                                             { spheres = value; };

    bool getShowFPS()                                                                                       { return showFPS; };
    void setShowFPS(bool value)                                                                             { showFPS = value; };
    
    bool getShowInfo()                                                                                      { return showInfo; };
    void setShowInfo(bool value)                                                                            { showInfo = value; };

    int getCamera()                                                                                         { return camera; };
    void setCamera(int value)                                                                               { camera = value; };

    int getCurrentScene()                                                                                   { return currentScene; };
    void setCurrentScene(int value)                                                                         { currentScene = value; };

    bool getSimulation()                                                                                    { return simulation; };
    void setSimulation(bool value)                                                                          { simulation = value; };

    float getSimulationTime()                                                                               { return simulationTime; };
    void setSimulationTime(float value)                                                                     { simulationTime = value; };

    int getSimulationMethod()                                                                               { return simulationMethod; };
    void setSimulationMethod(int value)                                                                     { simulationMethod = value; };
    
    bool getComputeGridEveryFrame()                                                                         { return computeGridEveryFrame; };
    void setComputeGridEveryFrame(bool value)                                                               { computeGridEveryFrame = value; };
    
    int getCurrentDevice()                                                                                  { return currentDevice; };
    void setCurrentDevice(int value)                                                                        { currentDevice = value; };

    int getCurrentDeviceLocalSize()                                                                         { return currentDeviceLocalSize; };
    void setCurrentDeviceLocalSize(int value)                                                               { currentDeviceLocalSize = value; };

    int getBuffersMemory()                                                                                  { return buffersMemory; };
    void setBuffersMemory(int value)                                                                        { buffersMemory = value; };

    bool getVortex()                                                                                        { return vortex; };
    void setVortex(bool value)                                                                              { vortex = value; };

    float getPointSize()                                                                                    { return pointSize; };
    void setPointSize(float value)                                                                          { pointSize = value; };

    int getReallocNeighboursBufferTime()                                                                    { return reallocNeighboursBufferTime; };
    void setReallocNeighboursBufferTime(int value)                                                          { reallocNeighboursBufferTime = value; };

    int getMergeSort1Time()                                                                                 { return mergeSort1Time; };
    void setMergeSort1Time(int value)                                                                       { mergeSort1Time = value; };

    int getMergeSort2Time()                                                                                 { return mergeSort2Time; };
    void setMergeSort2Time(int value)                                                                       { mergeSort2Time = value; };

    int getComputeGridTime()                                                                                { return computeGridTime; };
    void setComputeGridTime(int value)                                                                      { computeGridTime = value; };

    int getNeighboursTime()                                                                                 { return neighboursTime; };
    void setNeighboursTime(int value)                                                                       { neighboursTime = value; };

    int getComputeDensityTime()                                                                             { return computeDensityTime; };
    void setComputeDensityTime(int value)                                                                   { computeDensityTime = value; };

    int getComputeForceTime()                                                                               { return computeForceTime; };
    void setComputeForceTime(int value)                                                                     { computeForceTime = value; };
    
    int getIntegrateTime()                                                                                  { return integrateTime; };
    void setIntegrateTime(int value)                                                                        { integrateTime = value; };

    int getStateManagingTime()                                                                              { return stateManagingTime; };
    void setStateManagingTime(int value)                                                                    { stateManagingTime = value; };

    int getOverallComputationTime()                                                                         { return overallComputationTime; };
    void setOverallComputationTime(int value)                                                               { overallComputationTime = value; };

    int getDownloadTime()                                                                                   { return downloadTime; };
    void setDownloadTime(int value)                                                                         { downloadTime = value; };

    int getRenderingTime()                                                                                  { return renderingTime; };
    void setRenderingTime(int value)                                                                        { renderingTime = value; };

};