const int NUM_LIGHTS = 3;

uniform vec4 lightAmbient[NUM_LIGHTS];
uniform vec4 lightDiffuse[NUM_LIGHTS];
uniform vec4 lightSpecular[NUM_LIGHTS];

uniform vec4 materialAmbient;
uniform vec4 materialDiffuse;
uniform vec4 materialSpecular;
uniform vec4 materialEmission;

uniform float materialShininess;

varying vec3 _eyeVec;
varying vec3 _lightVec[NUM_LIGHTS];
varying vec3 _normalVec;


void main() {
  vec3 normalVec = normalize(_normalVec);
  vec3 normalEye = normalVec;

  vec3 eyeVec = normalize(_eyeVec);

  vec4 matAmbient = materialAmbient;
  vec4 matDiffuse = materialDiffuse * gl_Color;
  vec4 matSpecular = materialSpecular; 
	
  vec3 finalColor = vec3(0); 

  for (int i = 0; i < NUM_LIGHTS; i++) {
	vec3 lightVec = normalize(_lightVec[i]);
    float NdotL = max(dot(normalVec, lightVec), 0.0);
      
    vec3 color = vec3(0.0); 

	if (NdotL != 0.0) {
	  color += vec3(matAmbient) * vec3(lightAmbient[i]);
	  color += vec3(matDiffuse) * vec3(lightDiffuse[i]) * NdotL;
	}
	else {
	  color += vec3(matAmbient) * vec3(lightAmbient[i]);
	}

	finalColor += color;
  }

  gl_FragColor = vec4(finalColor, 1.0);
}
