#pragma once

using namespace clframework;

class Window;

class SPHsim {

  private:

    CLSystem* clsys;

    CLLink* linkCPU;
    CLLink* linkGPU;
    
    // Common
    CLFunction* functionDistance2;
    CLFunction* functionLength2;
    CLFunction* functionZOrder;
    
    CLProgram* programReduceMaxFluidFluidCount;
    CLProgram* programReduceMaxFluidSolidCount;
    CLProgram* programReduceMaxSolidSolidCount;
    CLProgram* programComputeNeighboursFluidCount;
    CLProgram* programComputeNeighboursSolidCount;
    CLProgram* programCreateCellsFluid;
    CLProgram* programCreateCellsSolid;
    CLProgram* programCorrectCellsFluid;
    CLProgram* programCorrectCellsSolid;
    CLProgram* programMergeSortCellsFluid;
    CLProgram* programMergeSortCellsSolid;
    CLProgram* programMergeSortMergeCellsFluidA;
    CLProgram* programMergeSortMergeCellsFluidB;
    CLProgram* programMergeSortMergeCellsSolidA;
    CLProgram* programMergeSortMergeCellsSolidB;
    CLProgram* programSwapFluid;
    CLProgram* programSwapSolid;
    CLProgram* programClearCellIndicesFluid;
    CLProgram* programClearCellIndicesSolid;
    CLProgram* programCreateCellIndicesFluid;
    CLProgram* programCreateCellIndicesSolid;
    CLProgram* programFindNeighboursFluid;
    CLProgram* programFindNeighboursSolid;
    CLProgram* programFindNeighboursFluidN2;
    CLProgram* programFindNeighboursSolidN2;
    CLProgram* programComputeSolidVolume;
    CLProgram* programComputeDensity;
    CLProgram* programNormalizeDensityFirstPass;
    CLProgram* programNormalizeDensitySecondPass;
    CLProgram* programComputeForce;
    CLProgram* programIntegratePositions;
    CLProgram* programApplyXSPHFirstPass;
    CLProgram* programApplyXSPHSecondPass;
    CLProgram* programClearState;

    CLLocalBuffer* reduceScratchInt;
    CLLocalBuffer* reduceScratchFloat;
    CLLocalBuffer* mergeSortAux;

    CLGlobalBuffer* reduceResultInt;
    CLGlobalBuffer* reduceResultFloat;

    CLGlobalBuffer* density;
    CLGlobalBuffer* densityFluctuation;
    CLGlobalBuffer* isoDensity;
    CLGlobalBuffer* pressure;
    CLGlobalBuffer* vorticity;
    
    CLGlobalBuffer* pos;
    CLGlobalBuffer* posTmp;
    CLGlobalBuffer* vel;
    CLGlobalBuffer* velTmp;
    CLGlobalBuffer* velXSPH;
    CLGlobalBuffer* acc;
    CLGlobalBuffer* accTmp;

    CLGlobalBuffer* gridCellsFluidA;
    CLGlobalBuffer* gridCellsFluidB;
    CLGlobalBuffer* gridCellIndicesFluid;
    CLGlobalBuffer* mergeSortMergeCellsFluidLastCell;

    CLGlobalBuffer* gridCellsSolidA;
    CLGlobalBuffer* gridCellsSolidB;
    CLGlobalBuffer* gridCellIndicesSolid;
    CLGlobalBuffer* mergeSortMergeCellsSolidLastCell;

    CLGlobalBuffer* neighboursFluidFluid;
    CLGlobalBuffer* neighboursFluidFluidCount;
    CLGlobalBuffer* neighboursFluidSolid;
    CLGlobalBuffer* neighboursFluidSolidCount;
    CLGlobalBuffer* neighboursSolidSolid;
    CLGlobalBuffer* neighboursSolidSolidCount;

    CLGlobalBuffer* solidPos;
    CLGlobalBuffer* solidPosTmp;

    CLGlobalBuffer* solidVolume;

    CLKernelArgument* reduceLength;

    CLKernelArgument* dummyElements;
    CLKernelArgument* correctElements;

    CLKernelArgument* mergeSortMergeMergeSize;
    CLKernelArgument* mergeSortMergeTotalSize;

    CLKernelArgument* usePressure;
    CLKernelArgument* useViscosity;
    CLKernelArgument* useVorticity;
    CLKernelArgument* useVortex;
    CLKernelArgument* computePressure;
    CLKernelArgument* clearPressure;
    CLKernelArgument* neighboursFluidFluidCountMax;
    CLKernelArgument* neighboursFluidSolidCountMax;
    CLKernelArgument* neighboursSolidSolidCountMax;

    CLConstant* fluidParticlesCount;
    CLConstant* solidParticlesCount;
    CLConstant* deltaTime;
    CLConstant* deltaTime2;

    CLConstant* delta;
    CLConstant* omega;
    CLConstant* gravity;
    CLConstant* radius;
    CLConstant* radius2;
    CLConstant* invRadius;
    CLConstant* invRadius2;
    CLConstant* accelerationLimit;
    CLConstant* accelerationLimit2;
    CLConstant* wDensity;
    CLConstant* dWDensity;
    CLConstant* ddWViscosity;
    CLConstant* dWPressure;
    CLConstant* density0;
    CLConstant* mass0;
    CLConstant* viscosity0;
    CLConstant* solidViscosity0;
    CLConstant* stiffness;
    CLConstant* negativePressureFactor;
    CLConstant* seventh;
    CLConstant* surfaceTension;
    CLConstant* boundaryStiffness;
    CLConstant* vorticityFactor;
    CLConstant* vortexOrigin;
    CLConstant* vortexAxis;
    CLConstant* vortexFactor;

    // WCSPH
    CLProgram* programIntegrateVelocitiesWCSPH;


    // PCISPH || IISPH
    CLProgram* programIntegrateVelocities;
    CLProgram* programClearPressureForce;

    CLGlobalBuffer* pressureForce;


    // PCISPH
    CLProgram* programReduceMaxDensityFluctuation;
    CLProgram* programComputePressureForce;
    CLProgram* programUpdatePressures;
    CLProgram* programIntegrate;
    CLProgram* programSaveState;
    CLProgram* programLoadState;


    // IISPH
    CLProgram* programReduceSumDensityFluctuation;
    CLProgram* programPredictVelocity;
    CLProgram* programPredictDensity;
    CLProgram* programComputeD_ijp_j;
    CLProgram* programComputeSumA;
    CLProgram* programComputeDensityFluctuation;
    CLProgram* programComputePressureForceIisph;
    CLProgram* programIntegrateIisph;
    CLProgram* programClearIisphPrediction;
    CLProgram* programClearIisphPressures;

    CLGlobalBuffer* velocityPrediction;
    CLGlobalBuffer* densityPrediction;
    CLGlobalBuffer* d_ii;
    CLGlobalBuffer* a_ii;
    CLGlobalBuffer* d_ijp_j;
    CLGlobalBuffer* sumA;


    int minIterationsPCISPH;
    int minIterationsIISPH;
    int maxIterationsIISPH;
    float maxDensityFluctuation;
    float avgDensityFluctuation;
    float deltaTimeHost;

    float simTimeHost;
    int framesHost;

    float deltaHost;
    float omegaHost;
    float gravityHost;
    float radiusHost;
    float accelerationLimitHost;
    float wDensityHost;
    float dWDensityHost;
    float ddWViscosityHost;
    float dWPressureHost;
    float density0Host;
    float mass0Host;
    float viscosity0Host;
    float solidViscosity0Host;
    float volume0Host;
    float distance0Host;
    float stiffnessHost;
    float negativePressureFactorHost;
    float surfaceTensionHost;
    float boundaryStiffnessHost;
    float vorticityFactorHost;

    Vector<3, float> vortexOriginHost;
    Vector<3, float> vortexAxisHost;
    float vortexFactorHost;

    float restDistance;

    int reduceLocalSize;
    int mergeSortLocalSize;
    int mergeSortFluidGlobalSize;
    int mergeSortFluidDummyElements;
    int mergeSortFluidMergesCount;
    int mergeSortSolidGlobalSize;
    int mergeSortSolidDummyElements;
    int mergeSortSolidMergesCount;

    bool allCLProgramsBuilt;

    std::vector<Vector<3, float>> fluidParticles;
    std::vector<Vector<3, float>> solidParticles;

    Scene scene;

    Window* window;

    std::vector<bool (SPHsim::*)()> scenesPointers;

    float distance2(const Vector<3, float>& a, const Vector<3, float>& b);
    int getBuffersSize();
    std::vector<Vector<3, float>> createFluidBox(Vector<3, float> minCell, Vector<3, float> size, Vector<3, float> cut, float jitter, float radius = 10000);
    void initializeParticles(std::vector<Vector<3, float>>& fluidParticles, std::vector<Vector<3, float>>& solidParticles);
    void precomputeDelta();
    void reallocateNeighboursBuffers(bool neighboursCountComputed = false);
    void computeGridFluid();
    void computeGridSolid();

    bool createOpenCLObjects(CLLink* link);
    void deleteOpenCLObjects();

    bool setSceneDamBreak();
    bool setSceneDamBreakObstacle();
    bool setSceneDamBreakTwoFluids();
    bool setSceneOcean();
    bool setSceneOceanBig();
    bool setSceneDrop();
    bool setCurrentScene();

    void initScene();

  public:

    SPHsim(Window* wd);
    ~SPHsim();

    bool createScene();
    void resetScene();
    void deleteScene();

    bool nextScene();
    bool previousScene();

    void switchDevice();
    void switchSimulationMethod();

    Scene* getScene() { return &scene; };

    void doStep();

};