#include "../main.h"
 
Window::Window(): 
  hDC(NULL), 
  hRC(NULL), 
  hWnd(NULL), 
  sizeable(false),
  title("oclSPH"),
  errorTitle("Error"),
  active(true), 
  windowCreated(false) { 
  
  sphsim = new SPHsim(this);
  opengl = new OpenGL(this);
};

Window::~Window() {
  delete opengl;
  delete sphsim;
}

bool Window::createOpenGLWindow(bool sizeable) {
  Settings* settings = &Settings::getInstance();

  Log::getInstance()->writeLine(WndDebug, "Creating OpenGL window");

  this->sizeable = sizeable;
  windowCreated = false;

  HDC dc = GetDC(NULL);
  int widthDesktop = GetDeviceCaps(dc, HORZRES);
  int heightDesktop = GetDeviceCaps(dc, VERTRES);
  int colorDepthDesktop = GetDeviceCaps(dc, BITSPIXEL);
  ReleaseDC(NULL, dc);

	int PixelFormat;
	WNDCLASS wc;						
	DWORD	dwExStyle;				
	DWORD	dwStyle;				
	RECT WindowRect;				
	WindowRect.left = (long)0;		
  WindowRect.right = (long)settings->getWidth();		
	WindowRect.top = (long)0;			
	WindowRect.bottom = (long)settings->getHeight();		

	hInstance	= GetModuleHandle(NULL);			
	wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	
	wc.lpfnWndProc = (WNDPROC)WndProc;					
	wc.cbClsExtra = 0;								
	wc.cbWndExtra = 0;								
	wc.hInstance = hInstance;				
	wc.hIcon = LoadIcon(NULL, IDI_WINLOGO);	
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);	
	wc.hbrBackground = NULL;									
	wc.lpszMenuName = NULL;									
	wc.lpszClassName = title;						

	if (!RegisterClass(&wc)) {
    MessageBox(NULL, "Failed to register the window class.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		return false;											
	}
	
	if (settings->getFullscreen()) {
		DEVMODE dmScreenSettings;					
		memset(&dmScreenSettings, 0, sizeof(dmScreenSettings));	
		dmScreenSettings.dmSize = sizeof(dmScreenSettings);
		dmScreenSettings.dmPelsWidth = settings->getWidth();		
		dmScreenSettings.dmPelsHeight	= settings->getHeight();			
		dmScreenSettings.dmBitsPerPel	= settings->getColorDepth();			
		dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;

		if (ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL) {
      if (MessageBox(NULL, "The requested fullscreen mode is not supported by your video card. Use windowed mode instead?", title, MB_YESNO | MB_ICONEXCLAMATION) == IDYES) {
        settings->setFullscreen(false);
			}
			else {
				MessageBox(NULL, "Program will now close.", errorTitle, MB_OK | MB_ICONSTOP);
				return false;							
			}
		}
	}

  if (settings->getFullscreen()) {
		dwExStyle = WS_EX_APPWINDOW;		
		dwStyle = WS_SYSMENU | WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE;																
	}
	else {
    if (sizeable) {
      dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;	
		  dwStyle = WS_OVERLAPPEDWINDOW | WS_VISIBLE;		
    }
    else {
		  dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;	
		  dwStyle = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX;		
    }
	}

	AdjustWindowRectEx(&WindowRect, dwStyle, false, dwExStyle);		

  int widthWindow = WindowRect.right - WindowRect.left;
  int heightWindow = WindowRect.bottom - WindowRect.top;
  int posXWindow = (widthDesktop - widthWindow) / 2;
  int posYWindow = (heightDesktop - heightWindow) / 2;

  if (settings->getFullscreen()) {
    posXWindow = 0;
    posYWindow = 0;
  }

	if (!(hWnd = CreateWindowEx(dwExStyle,							
								title, // Nazov triedy
								title, // Titulok okna
								dwStyle | WS_CLIPSIBLINGS |	WS_CLIPCHILDREN, // Styl okna				
								posXWindow, posYWindow, // Pozicia okna
								widthWindow, // Sirka okna
								heightWindow, // Vyska okna
								NULL, // Rodicovske okno
								NULL, // Menu
								hInstance, // Instancia
								NULL))) { // Parameter na odoslanie pre WM_CREATE 
  
		killOpenGLWindow();
		MessageBox(NULL, "Window creation error.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		return false;							
	}

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR), // Velkost tohto Pixel Format Descriptora
		1,// Verzia
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL |	PFD_DOUBLEBUFFER, // Podporovane formaty					
		PFD_TYPE_RGBA, // RGBA format
		settings->getColorDepth(), // Farebna hlbka
		0, 0, 0, 0, 0, 0, // Ignorovane farebne bity
		0, // Alfa buffer
		0, // Ignorovany "shift" bit
		0, // Akumulacny buffer
		0, 0, 0, 0, // Ignorovane akumulacne bity
		16, // Z-Buffer (Depth Buffer)  
		0, // Stencil Buffer
		0, // Auxiliary Buffer
		PFD_MAIN_PLANE, // Hlavna vykreslovacia vrstva
		0, // Rezervovane
		0, 0, 0 // Ignorovane vrstvy masiek
	};
	
	if (!(hDC = GetDC(hWnd))) {
		killOpenGLWindow();					
		MessageBox(NULL, "Can't create a OpenGL Device Context.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		return false;			
	}

  std::ostringstream ss;
  ss.str("");

  ChooseAntiAliasingPixelFormat(PixelFormat, settings->getMSAA());
  if (PixelFormat == 0) {
    PixelFormat = ChoosePixelFormat(hDC, &pfd);
    ss << "Antialiasing wasn't set";
  }
  else {
    ss << "Antialiasing set to " << GetAntiAliasingPixelFormatString();
  }
  Log::getInstance()->writeLine(WndDebug, ss.str());

	if (PixelFormat == 0) {
		ReleaseDC(hWnd, hDC);				
		hDC = 0;										
		killOpenGLWindow();							
		hWnd = 0;		
    
    MessageBox(NULL, "Can't find a suitable PixelFormat.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		return false;													
	}

	if (!SetPixelFormat(hDC, PixelFormat, &pfd)) {
		killOpenGLWindow();			
		MessageBox(NULL, "Can't set the PixelFormat.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		return false;						
	}

	if (!(hRC = wglCreateContext(hDC))) {
		killOpenGLWindow();					
		MessageBox(NULL, "Can't create a OpenGL Rendering Context.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		return false;						
	}

	if (!wglMakeCurrent(hDC,hRC)) {
		killOpenGLWindow();							
		MessageBox(NULL, "Can't activate the OpenGL Rendering Context.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		return false;							
	} 

  if (glewInit() != GLEW_OK) {
    MessageBox(NULL, "Glew initialization failed.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
    return false;
  }

  if (settings->getMaximized() && sizeable) {
    ShowWindow(hWnd, SW_SHOWMAXIMIZED);	
  }
  else {
    ShowWindow(hWnd, SW_SHOW);	
  }

  RECT rect;
  if (GetClientRect(hWnd, &rect)) {
    settings->setWidth(rect.right - rect.left);
    settings->setHeight(rect.bottom - rect.top);
  }

  opengl->updateSize();

  LONG hIcon = (LONG)LoadIcon(GetModuleHandle(0), MAKEINTRESOURCE(101));
  SetClassLong(hWnd, GCL_HICON, hIcon);
	SetForegroundWindow(hWnd);		
	SetFocus(hWnd);

  opengl->createFonts();

  if (settings->getCamera() == 2) {
    showCursorCount = ShowCursor(false);
  }
  else {
    showCursorCount = ShowCursor(true);
  }

  windowCreated = true;

  return true;
}

void Window::killOpenGLWindow() {
  Settings* settings = &Settings::getInstance();

  std::ostringstream ss;
  ss << "Killing OpenGL window...";
  Log::getInstance()->writeLine(WndDebug, ss.str());

	if (settings->getFullscreen())	{
		ChangeDisplaySettings(NULL, 0);	
		showCursorCount = ShowCursor(true);					
	}

	if (hRC) {
		if (!wglMakeCurrent(NULL, NULL)) {
      MessageBox(NULL, "Release of DC and RC failed.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		}

		if (!wglDeleteContext(hRC)) {
      MessageBox(NULL, "Release Rendering Context failed.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		}

		hRC = NULL;										
	}

	if (hDC && !ReleaseDC(hWnd, hDC)) {
    MessageBox(NULL, "Release Device Context failed.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		hDC = NULL;								
	}

	if (hWnd && !DestroyWindow(hWnd)) {
    MessageBox(NULL,  "Could not release hWnd.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		hWnd = NULL;								
	}

	if (!UnregisterClass(title, hInstance)) {
    MessageBox(NULL, "Could not unregister class.", errorTitle, MB_OK | MB_ICONEXCLAMATION);
		hInstance = NULL;								
	}

  opengl->deleteFonts();
}

bool Window::restoreBeforeFullscreen() {
  Settings* settings = &Settings::getInstance();

  ChangeDisplaySettings(NULL, CDS_FULLSCREEN);

  return ChangeDisplaySettings(NULL, 0) == DISP_CHANGE_SUCCESSFUL;
}

void Window::toggleFullscreen() {
  Settings* settings = &Settings::getInstance();

  if (settings->getFullscreen()) {
    int widthWindow = settings->getWidth();
    int heightWindow = settings->getHeight();

    if (restoreBeforeFullscreen()) {
      settings->setFullscreen(!settings->getFullscreen());

      RECT rect;
      rect.left = 0;
      rect.top = 0;
      rect.right = widthWindow;
      rect.bottom = heightWindow;

      if (sizeable) {
        SetWindowLongPtr(hWnd, GWL_EXSTYLE, WS_EX_APPWINDOW | WS_EX_WINDOWEDGE);
        SetWindowLongPtr(hWnd, GWL_STYLE, WS_OVERLAPPEDWINDOW | WS_VISIBLE);
        AdjustWindowRect(&rect, WS_OVERLAPPEDWINDOW, FALSE);
      }
      else {
        SetWindowLongPtr(hWnd, GWL_EXSTYLE, WS_EX_APPWINDOW | WS_EX_WINDOWEDGE);
        SetWindowLongPtr(hWnd, GWL_STYLE, WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX | WS_POPUPWINDOW | WS_VISIBLE);
        AdjustWindowRect(&rect, WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX | WS_POPUPWINDOW, FALSE);
      }
    
      SetWindowPos(hWnd, HWND_TOP, 0, 0, rect.right-rect.left, rect.bottom-rect.top, SWP_SHOWWINDOW);
      
      if (settings->getMaximized() && sizeable) {
        ShowWindow(hWnd, SW_SHOWMAXIMIZED);	
      }     
      else {
        centerWindow();
      }
    }
    else {
      MessageBox(NULL, "Unfortunatelly fullscreen mode cannot be disabled.", errorTitle, MB_ICONEXCLAMATION);
    }
  }
  else {
    int widthWindow = settings->getWidth();
    int heightWindow = settings->getHeight();

    DEVMODE dm;
    dm.dmSize = sizeof(DEVMODE);
    dm.dmPelsWidth = widthWindow;
    dm.dmPelsHeight = heightWindow;
    dm.dmBitsPerPel = settings->getColorDepth();
    dm.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT | DM_BITSPERPEL;
    
    if (ChangeDisplaySettings(&dm, CDS_FULLSCREEN) == DISP_CHANGE_SUCCESSFUL) {
      settings->setFullscreen(!settings->getFullscreen());

      RECT rect;
      rect.left = 0;
      rect.top = 0;
      rect.right = widthWindow;
      rect.bottom = heightWindow;

      SetWindowLongPtr(hWnd, GWL_EXSTYLE, WS_EX_APPWINDOW);
      SetWindowLongPtr(hWnd, GWL_STYLE, WS_SYSMENU | WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE);
      SetWindowPos(hWnd, HWND_TOP, 0, 0, rect.right-rect.left, rect.bottom-rect.top, SWP_SHOWWINDOW);
    }
    else {
      MessageBox(NULL, "Unfortunatelly the requested fullscreen mode is not supported by your video card.", errorTitle, MB_ICONEXCLAMATION);
    }
  }
}

void Window::centerWindow() {
  Settings* settings = &Settings::getInstance();

  if (!settings->getFullscreen()) {
    RECT client, window;

    if (GetClientRect(hWnd, &client) && GetWindowRect(hWnd, &window)) {
      int bordersWidth = (window.right - window.left) - client.right;
      int bordersHeight = (window.bottom - window.top) - client.bottom;
      int widthWindow = settings->getWidth() + bordersWidth;
      int heightWindow = settings->getHeight() + bordersHeight;

      RECT desktop;
      const HWND hDesktop = GetDesktopWindow();
      GetWindowRect(hDesktop, &desktop);
      int widthDesktop = desktop.right;
      int heightDesktop = desktop.bottom;

      SetWindowPos(hWnd, HWND_TOP, (widthDesktop - widthWindow) / 2, (heightDesktop - heightWindow) / 2, widthWindow, heightWindow, SWP_SHOWWINDOW);
    }
  }
}

void Window::centerCursor() {
  Settings* settings = &Settings::getInstance();

  RECT client, window;

  if (GetClientRect(hWnd, &client) && GetWindowRect(hWnd, &window)) {
    int bordersWidth = (window.right - window.left) - client.right;
    int bordersHeight = (window.bottom - window.top) - client.bottom;

    int top = window.top + (bordersHeight - GetSystemMetrics(SM_CYSIZEFRAME));
    int left = window.left + (bordersWidth / 2);

    if (settings->getFullscreen()) {
      top = left = 0;
    }

    int x = left + (settings->getWidth()) / 2;
    int y = top + (settings->getHeight()) / 2;
    
    SetCursorPos(x, y);
  }  
}

int Window::getCenterCursorX() {
  Settings* settings = &Settings::getInstance();

  RECT client, window;

  if (GetClientRect(hWnd, &client) && GetWindowRect(hWnd, &window)) {
    int bordersWidth = (window.right - window.left) - client.right;
    return (settings->getWidth() + bordersWidth) / 2;
  }

  return 0;
}

int Window::getCenterCursorY() {
  Settings* settings = &Settings::getInstance();

  RECT client, window;

  if (GetClientRect(hWnd, &client) && GetWindowRect(hWnd, &window)) {
    int bordersHeight = (window.bottom - window.top) - client.bottom;
    return (settings->getHeight() + bordersHeight) / 2;
  }

  return 0;
}

bool Window::start() {
  opengl->renderInitInfo("Initializing OpenGL ...");
  if (!opengl->initOpenGL()) {
    MessageBox(NULL, "Error occured while initializing OpenGL!", errorTitle, MB_OK | MB_ICONEXCLAMATION);
    return false;
  }

  opengl->renderInitInfo("Initializing OpenCL ...");
  if (!sphsim->createScene()) {
    MessageBox(NULL, "Error occured while creating scene!", errorTitle, MB_OK | MB_ICONEXCLAMATION);
    return false;
  }

  return true;
}

void Window::end() {
  opengl->freeOpenGL();
  sphsim->deleteScene();
}

void Window::doMainLoop() {
  if (windowCreated) {
    Settings* settings = &Settings::getInstance();
   
    if (settings->getSimulation()) {
      sphsim->doStep();
    }

    double t = clock();
    opengl->render(sphsim->getScene());
    settings->setRenderingTime(clock() - t);

    SwapBuffers(hDC);
  }
}

void Window::updateWindowSize() { 
  if (windowCreated) {
    opengl->updateSize(); 
  }
}

void Window::keyDown(int key) { 
  if (windowCreated) {
    keys[key] = true; 
    opengl->keyDown(key);
  }
}

void Window::keyUp(int key) { 
  if (windowCreated) {
    Settings* settings = &Settings::getInstance();
    keys[key] = false; 

    switch (key) {

      case VK_DIVIDE: {
        float pointSize = settings->getPointSize() - 1;

        if (pointSize >= 1) {
          settings->setPointSize(pointSize);
        }
        break;
      }

      case VK_MULTIPLY: {
        float pointSize = settings->getPointSize() + 1;

        if (pointSize <= 15) {
          settings->setPointSize(pointSize);
        }
        break;
      }

      case VK_SPACE: {
        settings->setSimulation(!settings->getSimulation());
        break;
      }

      case VK_PRIOR: {
        settings->setSimulation(false);
        opengl->renderInitInfo("Switching scene ...");
        sphsim->nextScene();
        break;
      }

      case VK_NEXT: {
        settings->setSimulation(false);
        opengl->renderInitInfo("Switching scene ...");
        sphsim->previousScene();
        break;
      }

      case 'C': {
        settings->setCamera((settings->getCamera() + 1) % 3);
        
        if (settings->getCamera() == 2) {
          while ((showCursorCount = ShowCursor(false)) >= 0); 
        }
        else {
          while ((showCursorCount = ShowCursor(true)) < 0); 
        }

        break;
      }

      case 'D': {
        settings->setSimulation(false);
        opengl->renderInitInfo("Switching OpenCL device ...");
        sphsim->switchDevice();
        break;
      }
      
      case 'F': {
        settings->setShowFPS(!settings->getShowFPS());
        break;
      }

      case 'G': {
        settings->setComputeGridEveryFrame(!settings->getComputeGridEveryFrame());
        break;
      }

      case 'H': {
        settings->setShowAxes(!settings->getShowAxes());
        break;
      }

      case 'I': {
        settings->setShowInfo(!settings->getShowInfo());
        break;
      }

      case 'L': {
        settings->setShowLights(!settings->getShowLights());
        break;
      }

      case 'M': {
        settings->setSimulation(false);
        opengl->renderInitInfo("Switching simulation method ...");
        sphsim->switchSimulationMethod();
        break;
      }

      case 'O': {
        settings->setShowSolidParticles(!settings->getShowSolidParticles());
        break;
      }

      case 'P': {
        settings->setFluidParticlesVisualization((settings->getFluidParticlesVisualization() + 1) % 3);
        break;
      }

      case 'R': {
        settings->setSimulation(false);
        sphsim->resetScene();
        break;
      }

      case 'S': {
        settings->setSpheres(!settings->getSpheres());
        break;
      }

      case 'V': {
        settings->setVortex(!settings->getVortex());
        break;
      }

      case 'W': {
        settings->setShowWireframe(!settings->getShowWireframe());
        if (settings->getShowWireframe()) {
          glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
          glDisable(GL_CULL_FACE);
        }
        else {
          glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
          glEnable(GL_CULL_FACE);
        }
        break;
      }  

    }

    opengl->keyUp(key);
  }
}

bool Window::getCursorPosWindow(int* x, int* y) { 
  POINT cursor;

  if (GetCursorPos(&cursor)) {
    ScreenToClient(hWnd, &cursor);

    *x = cursor.x;
    *y = cursor.y-4; // Visual Studio 2013 bug hack

    return true;
  }

  return false;
}

void Window::mouseDown(int mouseButton, int x, int y) { 
  if (windowCreated) {
    for (int i = 0; i < 8; i++) {
      int button = (int)(log((float)(mouseButton & (int)pow(2.0f, i))) / log(2.0f));
      mouse[button].setPressed(true);
      mouse[button].setX(x);
      mouse[button].setY(y);
    }
    opengl->mouseDown(mouseButton, x, y);
  }
}

void Window::mouseUp(int mouseButton, int x, int y) { 
  if (windowCreated) {
    for (int i = 0; i < 8; i++) {
      int button = (int)(log((float)(mouseButton & (int)pow(2.0f, i))) / log(2.0f));
      mouse[button].setPressed(false);
      mouse[button].setX(x);
      mouse[button].setY(y);
    }
    opengl->mouseUp(mouseButton, x, y);
  }
}

void Window::mouseMove(int mouseButton, int x, int y) { 
  if (windowCreated) {
    opengl->mouseMove(mouseButton, x, y);
  }
}

void Window::mouseScroll(float delta) { 
  if (windowCreated) {
    opengl->mouseScroll(delta / 3000.0f); 
  }
}