#pragma once

class Window {

  private:

    HDC hDC;		
    HGLRC hRC;		
    HWND hWnd;		
    HINSTANCE	hInstance;		

    bool sizeable;
    LPCTSTR title;
    LPCTSTR errorTitle;

    int showCursorCount;

    bool active;	
    bool windowCreated;

    bool keys[256];	
    Mouse mouse[128];

    SPHsim* sphsim;
    OpenGL* opengl;

  public:

    Window();
    ~Window();

    bool createOpenGLWindow(bool sizeable);
    void killOpenGLWindow();

    bool restoreBeforeFullscreen();
    void toggleFullscreen();
    void centerWindow();
    void centerCursor();
    
    int getCenterCursorX();
    int getCenterCursorY();

    bool start();
    void end();

    void doMainLoop();
    void updateWindowSize();
    
    bool getKeyAsync(int key)                                                                               { return (bool)GetAsyncKeyState(key); };
    bool getKey(int key)                                                                                    { return keys[key]; };
    void keyDown(int key);
    void keyUp(int key);

    bool getCursorPosWindow(int* x, int* y);

    Mouse& getMouse(int mouseButton)                                                                        { return mouse[mouseButton]; };
    void mouseDown(int mouseButton, int x, int y);
    void mouseUp(int mouseButton, int x, int y);
    void mouseMove(int mouseButton, int x, int y);
    void mouseScroll(float delta);

    HDC getHDC()                                                                                            { return hDC; };
    HWND getHWND()                                                                                          { return hWnd; };

    bool getActive()                                                                                        { return active; };
    void setActive(bool value)                                                                              { active = value; };

    bool isSizeable()                                                                                       { return sizeable; };
    bool isWindowCreated()                                                                                  { return windowCreated; };

};